import tempfile
import os
import time

import tator

from._common import assert_vector_equal

def test_clone_multi(host, token, project, multi_type, multi):
    tator_api = tator.get_api(host, token)
    multi_obj = tator_api.get_media(multi)
    response = tator_api.clone_media_list(project=project,
                                          media_id=[multi],
                                          clone_media_spec={'dest_project': project,
                                                            'dest_section': 'Cloned multi',
                                                            'dest_type': multi_type})
    assert isinstance(response, tator.models.CreateListResponse)
    assert len(response.id) == 1

def test_clone_videos(host, token, project, video_type, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    response = tator_api.clone_media_list(project=project,
                                          media_id=[video],
                                          clone_media_spec={'dest_project': project,
                                                            'dest_section': 'Cloned video',
                                                            'dest_type': video_type})
    assert isinstance(response, tator.models.CreateListResponse)
    assert len(response.id) == 1

    cloned_video = tator_api.get_media(response.id[0])
    with tempfile.TemporaryDirectory() as temp_dir:
        outpath = os.path.join(temp_dir, "video.mp4")
        for progress in tator.download_media(tator_api, cloned_video, outpath):
            print(f"Cloned video download progress: {progress}%")
        assert(os.path.exists(outpath))

def test_clone_images(host, token, project, image_type, image):
    tator_api = tator.get_api(host, token)
    image_obj = tator_api.get_media(image)

    response = tator_api.clone_media_list(project=project,
                                          media_id=[image],
                                          clone_media_spec={'dest_project': project,
                                                            'dest_section': 'Cloned image',
                                                            'dest_type': image_type})
    assert isinstance(response, tator.models.CreateListResponse)
    assert len(response.id) == 1

    cloned_image = tator_api.get_media(response.id[0])
    with tempfile.TemporaryDirectory() as temp_dir:
        outpath = os.path.join(temp_dir, "image.jpg")
        for progress in tator.download_media(tator_api, cloned_image, outpath):
            print(f"Cloned image download progress: {progress}%")
        assert(os.path.exists(outpath))

def test_clone_multi_util_same_host(host, token, project, multi_type, multi):
    tator_api = tator.get_api(host, token)
    query_params = {'project': project, 'media_id': [multi]}
    section = 'Cloned media util same host'
    created_ids = []
    generator = tator.util.clone_media_list(tator_api, query_params, project, {}, multi_type,
                                            section)
    for num_created, num_total, response, id_map in generator:
        print(f"Created {num_created} of {num_total} files...")
        created_ids.append(response.id)
    print(f"Finished creating {num_created} files!")
    assert len(created_ids) == 1

def test_clone_videos_util_same_host(host, token, project, video_type, video):
    tator_api = tator.get_api(host, token)
    query_params = {'project': project, 'media_id': [video]}
    section = 'Cloned media util same host'
    created_ids = []
    generator = tator.util.clone_media_list(tator_api, query_params, project, {}, video_type,
                                            section)
    for num_created, num_total, response, id_map in generator:
        print(f"Created {num_created} of {num_total} files...")
        created_ids.append(response.id)
    print(f"Finished creating {num_created} files!")
    assert len(created_ids) == 1

def test_clone_images_util_same_host(host, token, project, image_type, image):
    tator_api = tator.get_api(host, token)
    query_params = {'project': project, 'media_id': [image]}
    section = 'Cloned media util same host'
    created_ids = []
    generator = tator.util.clone_media_list(tator_api, query_params, project, {}, image_type,
                                            section)
    for num_created, num_total, response, id_map in generator:
        print(f"Created {num_created} of {num_total} files...")
        created_ids.append(response.id)
    print(f"Finished creating {num_created} files!")
    assert len(created_ids) == 1

def test_clone_multi_util_different_host(host, token, project, multi_type, multi, video):
    tator_api = tator.get_api(host, token)
    dest_api = tator.get_api(host, token)
    assert tator_api is not dest_api
    query_params = {'project': project, 'media_id': [multi]}
    section = 'Cloned media util different host'
    created_ids = []
    generator = tator.util.clone_media_list(tator_api, query_params, project, {video:video},
                                            multi_type, section, dest_api)
    for num_created, num_total, response, id_map in generator:
        print(f"Created {num_created} of {num_total} files...")
        created_ids.append(response.id)
    print(f"Finished creating {num_created} files!")
    tator.util.clone_media_list(tator_api, query_params, project, multi_type,
                                'Cloned media util different host', tator_api)
    assert len(created_ids) == 1

def test_clone_videos_util_different_host(host, token, project, video_type, video):
    tator_api = tator.get_api(host, token)
    dest_api = tator.get_api(host, token)
    assert tator_api is not dest_api
    query_params = {'project': project, 'media_id': [video]}
    section = 'Cloned media util different host'
    created_ids = []
    generator = tator.util.clone_media_list(tator_api, query_params, project, {}, video_type,
                                            section, dest_api)
    for num_created, num_total, response, id_map in generator:
        print(f"Created {num_created} of {num_total} files...")
        created_ids.append(response.id)
    print(f"Finished creating {num_created} files!")
    tator.util.clone_media_list(tator_api, query_params, project, video_type,
                                'Cloned media util different host', tator_api)
    assert len(created_ids) == 1

def test_clone_images_util_different_host(host, token, project, image_type, image):
    tator_api = tator.get_api(host, token)
    dest_api = tator.get_api(host, token)
    assert tator_api is not dest_api
    query_params = {'project': project, 'media_id': [image]}
    section = 'Cloned media util different host'
    created_ids = []
    generator = tator.util.clone_media_list(tator_api, query_params, project, {}, image_type,
                                            section, dest_api)
    for num_created, num_total, response, id_map in generator:
        print(f"Created {num_created} of {num_total} files...")
        created_ids.append(response.id)
    print(f"Finished creating {num_created} files!")
    assert len(created_ids) == 1



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class LeafType(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attribute_types': 'list[AttributeType]',
        'description': 'str',
        'dtype': 'str',
        'id': 'int',
        'name': 'str',
        'project': 'int'
    }

    attribute_map = {
        'attribute_types': 'attribute_types',
        'description': 'description',
        'dtype': 'dtype',
        'id': 'id',
        'name': 'name',
        'project': 'project'
    }

    def __init__(self, attribute_types=None, description='', dtype=None, id=None, name=None, project=None, local_vars_configuration=None):  # noqa: E501
        """LeafType - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attribute_types = None
        self._description = None
        self._dtype = None
        self._id = None
        self._name = None
        self._project = None
        self.discriminator = None

        if attribute_types is not None:
            self.attribute_types = attribute_types
        if description is not None:
            self.description = description
        if dtype is not None:
            self.dtype = dtype
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if project is not None:
            self.project = project

    @property
    def attribute_types(self):
        """
        Attribute type definitions.

        :return: The attribute_types of this LeafType. 
        :rtype: list[AttributeType]
        """
        return self._attribute_types

    @attribute_types.setter
    def attribute_types(self, attribute_types):
        """
        Attribute type definitions.

        :param attribute_types: The attribute_types of this LeafType.
        :type: list[AttributeType]
        """

        self._attribute_types = attribute_types

    @property
    def description(self):
        """
        Description of the leaf type.

        :return: The description of this LeafType. 
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Description of the leaf type.

        :param description: The description of this LeafType.
        :type: str
        """

        self._description = description

    @property
    def dtype(self):
        """
        Name of this data type, value is always \"leaf\".

        :return: The dtype of this LeafType. 
        :rtype: str
        """
        return self._dtype

    @dtype.setter
    def dtype(self, dtype):
        """
        Name of this data type, value is always \"leaf\".

        :param dtype: The dtype of this LeafType.
        :type: str
        """

        self._dtype = dtype

    @property
    def id(self):
        """
        Unique integer identifying a leaf type.

        :return: The id of this LeafType. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying a leaf type.

        :param id: The id of this LeafType.
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """
        Name of the leaf type.

        :return: The name of this LeafType. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the leaf type.

        :param name: The name of this LeafType.
        :type: str
        """

        self._name = name

    @property
    def project(self):
        """
        Unique integer identifying project for this leaf type.

        :return: The project of this LeafType. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying project for this leaf type.

        :param project: The project of this LeafType.
        :type: int
        """

        self._project = project

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LeafType):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, LeafType):
            return True

        return self.to_dict() != other.to_dict()



import glob
import logging
import os
import shutil
import tempfile
import time
import uuid
from textwrap import dedent

import pytest

import tator
from tator.util._upload_file import _upload_file

logger = logging.getLogger(__name__)

def _create_yaml_file_str() -> str:
    """ Creates a argo manifest file used by the unit tests in this file
    """

    return dedent("""\
apiVersion: argoproj.io/v1alpha1
kind: Workflow
metadata:
  generateName: test-algorithm-launcher-
spec:
  entrypoint: pipeline
  ttlStrategy:
    secondsAfterCompletion: 300 # Time to live after workflow is completed, replaces ttlSecondsAfterFinished
    secondsAfterSuccess: 300 # Time to live after workflow is successful
    secondsAfterFailure: 600 # Time to live after workflow fails
  volumeClaimTemplates:
  - metadata:
      name: workdir
    spec:
      accessModes: [ "ReadWriteOnce" ]
      storageClassName: nfs-client
      resources:
        requests:
          storage: 100Mi
  templates:
  - name: pipeline
    steps:
    - - name: test
        template: test
  - name: test
    script:
      image: cvisionai/tator_transcoder #localhost:5000/tator_transcoder
      env:
      - name: TATOR_MEDIA_IDS
        value: "{{workflow.parameters.media_ids}}"
      - name: TATOR_API_SERVICE
        value: "{{workflow.parameters.rest_url}}"
      - name: TATOR_AUTH_TOKEN
        value: "{{workflow.parameters.rest_token}}"
      - name: TATOR_PROJECT_ID
        value: "{{workflow.parameters.project_id}}"
      - name: TATOR_WORK_DIR
        value: "/work"
      resources:
        limits:
          cpu: 1000m
          memory: 500Mi
      volumeMounts:
      - name: workdir
        mountPath: /work
      command: [python3]
      source: |
        #!/usr/bin/env python3

        import os
        import tator

        def main():
            host = os.path.dirname(os.getenv('TATOR_API_SERVICE'))
            token = os.getenv('TATOR_AUTH_TOKEN')
            project = int(os.getenv('TATOR_PROJECT_ID'))
            media_ids = os.getenv('TATOR_MEDIA_IDS')
            media_ids = [int(m) for m in media_ids.split(',')]

            tator_api = tator.get_api(host=host, token=token)

            print(f'{host} {token} {project} {media_ids}')

            for media in media_ids:
                name = f'test_media_{media}'
                dq = f'test_media_id:{media}'
                spec = tator.models.AnalysisSpec(name=name, data_query=dq)
                _ = tator_api.create_analysis(project=project, analysis_spec=spec)

        if __name__ == '__main__':
            main()
        """)

def _create_workflow_manifest(
        host: str,
        token: str,
        project: int) -> str:
    """ Creates the argo workflow manifest file used by the tests in this module

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project

    Returns:
        URL to uploaded and saved algorithm manifest file
    """
    tator_api = tator.get_api(host=host, token=token)

    fd, local_yaml_file = tempfile.mkstemp()
    try:
        with os.fdopen(fd, 'w') as file_handle:
            file_handle.write(_create_yaml_file_str())

        for progress, upload_info in _upload_file(tator_api, project, local_yaml_file):
            pass
        url = tator_api.get_download_info(project, {'keys': [upload_info.key]})[0].url

        # Save the uploaded file using the save algorithm manifest endpoint
        spec = tator.models.AlgorithmManifestSpec(name='workflow.yaml', upload_url=url)
        response = tator_api.save_algorithm_manifest(project=project, algorithm_manifest_spec=spec)

    finally:
        os.remove(local_yaml_file)

    return response.url

@pytest.fixture(scope='module')
def algorithm_name(request, project: int) -> str:
    """ Pytest fixture used to create a registered algorithm used by the tests in this module

    Yields:
        Unique algorithm name to be used for the registered algorithms in this test
        Deletes the registered algorithm once context switches back to this method
    """

    import uuid
    import tator

    # Get the user ID
    host = request.config.option.host
    token = request.config.option.token
    tator_api = tator.get_api(host=host, token=token)
    user = tator_api.whoami()

    # Create the workflow manifest that will be used when testing the algorithm launch endpoint
    manifest_url = _create_workflow_manifest(host=host, token=token, project=project)

    # Register the algorithm using the unique name
    algorithm_name = 'test_algorithm_launch_workflow'

    spec = tator.models.Algorithm(
        name=algorithm_name,
        project=project,
        user=user.id,
        description=None,
        manifest=manifest_url,
        cluster=None,
        files_per_job=100)

    response = tator_api.register_algorithm(project=project, algorithm_spec=spec)

    yield algorithm_name

    # Once we're done with the tests in this module, delete the algorithm
    # (and subsequently its unique name)
    _ = tator_api.delete_algorithm(id=response.id)

def _assert_algorithm_workflow_results(
    tator_api: tator.api,
    project: int,
    expected_analysis_count: int) -> None:
    """ Polls the tator database waiting for the argo workflow to complete its database inserts

    Performs assert on if the count of analysis entries matches the provided expected count

    Args:
        tator_api: Interface to tator
        project: Unique identifier of test project
        expected_analysis_count: Expected number of analysis entries with test_media_id data query
    """

    MAX_RETRIES = 10
    num_retries = 0
    matched_analysis_count = False

    while True:

        analysis_list = tator_api.get_analysis_list(project=project)
        analysis_count = 0
        log_msg = f'num_retries: {num_retries}'
        print(log_msg)
        for analysis in analysis_list:
            if 'test_media_id' in analysis.data_query:
                print(str(analysis))
                analysis_count += 1

        matched_analysis_count = analysis_count == expected_analysis_count
        if matched_analysis_count:
            break

        num_retries += 1
        if num_retries < MAX_RETRIES:
            time.sleep(10)
        else:
            log_msg = f'Number of detected analysis entries from workflow: {analysis_count}'
            logger.error(log_msg)
            logger.error('Reached maximum retries')
            break

    assert matched_analysis_count

def test_algorithm_launch(
        host: str,
        token: str,
        project: int,
        algorithm_name: str,
        image_type: int,
        image_set: str) -> None:
    """ Unit tests the LaunchAlgorithm endpoint

    Performs the following unit tests:
    - Add a bunch of images, register the workflow that writes entries to the Analysis model
      and check to see if the number of entries match in a given amount of processing time.
    - Provide a specific media list, launch the workflow, 
      and perform the analysis entry counting as above
    - Provide a blank media list and ensure no workflow was launched (using the uid)
    - Provide a media query, launch the workflow,
      and perform the analysis entry counting as above 

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
        algorithm_name: Unique algorithm name of a registered workflow to be used by this test
        image_type: Unique identifier of the media type related to images saved to this project
        image_set: List of paths to temporary images that can be uploaded to tator
    """

    tator_api = tator.get_api(host=host, token=token)
    number_of_media = 10

    # Get the media that's already in the project (perhaps added from other tests)
    medias = tator_api.get_media_list(project=project)
    number_of_media += len(medias)

    # Add some media to the project
    paths = glob.glob(image_set + '/**/*.jpg', recursive=True)
    paths = [os.path.join(image_set, path) for path in paths]
    paths = paths[:number_of_media]

    for image_file in paths:
        for progress, response in tator.util.upload_media(tator_api, image_type, image_file):
            log_msg = f"Upload image {image_file} - progress: {progress}%"
            logger.info(log_msg)

    num_retries = 0
    max_retries = 30
    while True:
        medias = tator_api.get_media_list(project=project)
        if len(medias) == number_of_media:
            break

        num_retries += 1
        if num_retries >= max_retries:
            break
        time.sleep(0.5)

    assert len(medias) == number_of_media

    # Now, launch the algorithm using all the media in the project
    spec = tator.models.AlgorithmLaunchSpec(algorithm_name=algorithm_name)
    response = tator_api.algorithm_launch(project=project, algorithm_launch_spec=spec)
    assert len(response.uid) == 1

    # The algorithm workflow doesn't do much but saves an analysis object for each media ID
    # that was passed along to it in the data query field. So we assume the workflow
    # will be done soon (tm) and these analysis entries should pop up. Count them up and
    # once we have one entry per media, we're good to go.
    expected_analysis_count = number_of_media
    _assert_algorithm_workflow_results(
        tator_api=tator_api,
        project=project,
        expected_analysis_count=expected_analysis_count)

    # Now, launch the algorithm again with an arbitrary set of media and test the results
    media_ids = [medias[0].id, medias[1].id]
    print(f"Providing following media list: {media_ids}")
    expected_analysis_count += len(media_ids)
    spec = tator.models.AlgorithmLaunchSpec(algorithm_name=algorithm_name, media_ids=media_ids)
    response = tator_api.algorithm_launch(project=project, algorithm_launch_spec=spec)
    assert len(response.uid) == 1
    _assert_algorithm_workflow_results(
        tator_api=tator_api,
        project=project,
        expected_analysis_count=expected_analysis_count)

    # Next, launch the algorithm again with a blank set of media ids and test the results
    # - This shouldn't launch the argo workflow
    media_ids = []
    expected_analysis_count += len(media_ids)
    spec = tator.models.AlgorithmLaunchSpec(algorithm_name=algorithm_name, media_ids=media_ids)
    response = tator_api.algorithm_launch(project=project, algorithm_launch_spec=spec)
    assert len(response.uid) == 0

    # Use the media query to get media of a specific name
    target_name = os.path.basename(paths[0])
    medias = tator_api.get_media_list(project=project, name=target_name)
    print(f"Providing following media name query: {target_name} (Medias matching this name: {len(medias)})")

    spec = tator.models.AlgorithmLaunchSpec(algorithm_name=algorithm_name, media_query=f"?name={target_name}")
    response = tator_api.algorithm_launch(project=project, algorithm_launch_spec=spec)
    assert len(response.uid) == 1
    expected_analysis_count += len(medias)
    _assert_algorithm_workflow_results(
        tator_api=tator_api,
        project=project,
        expected_analysis_count=expected_analysis_count)



import os
import subprocess
import tempfile
import traceback
import shutil
import sys
import uuid


from collections import defaultdict
import numpy as np

try:
    import cv2
except:
    print("Unable to import cv2, required for extractor")
    traceback.print_exc()
    sys.exit(-1)


import tator
from tator.transcode.make_thumbnails import make_thumbnails
from tator.transcode.transcode import make_video_definition
from tator.transcode.transcode import convert_streaming
from tator.transcode.transcode import default_archival_upload

from tator.openapi.tator_openapi.models import CreateResponse



def import_media(api,
                 project,
                 media_type_id,
                 path,
                 fname,
                 section,
                 upload_gid,
                 work_dir):

    md5sum = tator.util.md5sum(path)
    response = api.create_media(project,
                                {'type': media_type_id,
                                 'section': section,
                                 'name': fname,
                                 'md5': md5sum,
                                 'gid': upload_gid,
                                 'uid': str(uuid.uuid1())})
    assert isinstance(response, CreateResponse)
    media_id = response.id

    # Peel apart api to get host/token combo (TODO: not great)
    host = api.api_client.configuration.host
    token = api.api_client.configuration.api_key['Authorization']
    with tempfile.TemporaryDirectory(dir=work_dir) as td:
        try:
            thumb_path = os.path.join(td, f"{uuid.uuid4()}.jpg")
            thumb_gif_path = os.path.join(td, f"{uuid.uuid4()}.gif")
            make_thumbnails(host, token, media_id,
                            path, thumb_path, thumb_gif_path)

            video_definition = make_video_definition(path)
            video_height = video_definition["resolution"][0]
            video_width = video_definition["resolution"][1]

            # Now transcode to streaming at a fixed resolution
            convert_streaming(host,
                              token,
                              media_id,
                              path,
                              td,
                              video_width,
                              video_height,
                              [f"{video_height}:23:h264"])

            default_archival_upload(api,
                                    host,
                                    media_id,
                                    path,
                                    True)
        except Exception as e:
            print(f"Error {e}")
            api.delete_media(media_id)
            traceback.print_exc()
            return None

    return media_id

def process_file(api,
                 project,
                 media_file,
                 media_el,
                 mode,
                 metadata,
                 output_dir,
                 tator_section_output=None,
                 output_type_id=None,
                 upload_gid=str(uuid.uuid1()),
                 work_dir=None):
    """
    Run the extractor on on a given file. Can optionally upload the extraction
    data to Tator if `tator_section_output` and `output_type_id` are given. Else
    the extractions are placed in `output_dir` for further use.

    :param api: Iniitalized :class:`tator.TatorApi` object
    :param project: Project ID
    :param media_file: Path to local copy of media file
    :param media_el: :class:`tator.models.Media` Media element
    :param mode: str choice of
                 - state : Whole frame capture based on state's frame
                 - localization_keyframe : Whole frame on localization frame
                 - localization_thumbnail : Thumbnail of localization
                 - track_thumbnail : Thumbnail of track (mp4)
    :param metadata: list of metadata to process for this media on a given mode
    :param output_dir: path to output extraction files
    :param tator_section_output: Name of section to output to in project
                                 (optional)
    :param output_type_id: Type ID of media to use for extraction data
                           (optional)
    :param upload_gid: str representation of a unique task id (optional)
    """
    grouped_by_frame = defaultdict(lambda: [])
    if mode == "track_thumbnail":

        # First get the localization_type id
        random_local = api.get_localization(metadata[0].localizations[0])
        localization_type = random_local.meta
        localizations = api.get_localization_list(project,
                                                  media_id=[media_el.id],
                                                  type=localization_type)
        localizations = [x.to_dict() for x in localizations]
        for l in localizations:
            found = False
            for entry in metadata:
                found = True
                l['track'] = entry.id
            if found:
                grouped_by_frame[l['frame']].append(l)
    else:
        for entry in metadata:
            frame = entry.frame
            grouped_by_frame[frame].append(entry.to_dict())

    vid = cv2.VideoCapture(media_file)
    if cv2.__version__ >= "3.2.0":
        vid_len = int(vid.get(cv2.CAP_PROP_FRAME_COUNT))
    else:
        vid_len = int(vid.get(cv2.cv.CV_CAP_PROP_FRAME_COUNT))

    frame_num = 0
    if len(grouped_by_frame.keys()) > 0:
        max_frame = np.max(list(grouped_by_frame.keys()))
    else:
        print("Nothing to extract")
        max_frame = 0

    fail_count = 0
    while frame_num <= max_frame:
        ok, image = vid.read()
        if not ok:
            fail_count += 1
            if fail_count >= 10:
                raise RuntimeError("Failed to grab video frame")
        else:
            fail_count = 0
            if frame_num in grouped_by_frame:
                print(f"Extracting metadata from {media_file}:{frame_num} of {max_frame}")
                if mode == 'state' or mode == 'localization_keyframe':
                    output_name = media_el.name
                    output_name += f"_{frame_num}.png"
                    output_fp = os.path.join(output_dir, output_name)
                    cv2.imwrite(output_fp, image)
                else:
                    _extract_thumbnails(image,
                                        grouped_by_frame[frame_num],
                                        output_dir)
            else:
                pass #print(f"Skipping {frame_num} of {max_frame}")
        frame_num += 1

    # Handle uploads
    if mode == 'track_thumbnail':
        print("Generating track thumbnails")
        for track in metadata:
            print(f"Making track {track.id}")
            track_dir = os.path.join(output_dir,
                                     str(track.id))
            os.makedirs(track_dir)
            local_ids = track.localizations
            track_locals = [local for local in localizations if local['id'] in local_ids]
            track_locals.sort(key=lambda x: x['frame'])
            for idx, track_local in enumerate(track_locals):
                shutil.copyfile(os.path.join(output_dir, f"{track_local['id']}.png"),
                                os.path.join(track_dir, f"{idx:05d}.png"))
            ffmpeg_cmd = ["ffmpeg",
                          "-r", "1",
                          "-v", "error",
                          "-f", "image2",
                          "-i", os.path.join(track_dir, "%05d.png"),
                          "-crf", "23",
                          os.path.join(track_dir, "clip.mp4")
                          ]
            subprocess.run(ffmpeg_cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

            print(f"Uploading Clip for {track.id}")
            if tator_section_output and output_type_id:
                # Upload the media and update the track object
                media_id = import_media(api,
                                        project,
                                        output_type_id,
                                        os.path.join(track_dir, "clip.mp4"),
                                        fname=f"{track.id}.mp4",
                                        section=tator_section_output,
                                        upload_gid=upload_gid,
                                        work_dir=work_dir)
                if media_id:
                    api.update_state(track.id, {"attributes": {"_extracted": media_id}})
                    print(f"Uploaded Track({track.id}) Extraction to {media_id}")
    elif mode == 'localization_thumbnail' and tator_section_output and output_type_id:
        for root, _, files in os.walk(output_dir):
            for fp in files:
                full_path = os.path.join(root, fp)
                for _, resp in tator.util.upload_media(api, output_type_id, full_path,
                                                       section=tator_section_output,
                                                       upload_gid=upload_gid):
                    pass
                try:
                    local_id = int(os.path.splitext(fp)[0])
                    media_id = resp.id
                    api.update_localization(local_id, {"attributes": {"_extracted": media_id}})
                except:
                    print(f"Unable to update '_extracted' attribute")
    else:
        if tator_section_output and output_type_id:
            ep = api.create_state_list if mode == 'state' else api.create_localization_list
            for root, _, files in os.walk(output_dir):
                for fp in files:
                    full_path = os.path.join(root, fp)
                    for _, resp in tator.util.upload_media(
                            api,
                            output_type_id,
                            full_path,
                            section=tator_section_output,
                            upload_gid=upload_gid):
                        pass
                    # Now duplicate state or localization(s) that caused the extraction
                    # into new media
                    media_id = resp.id
                    frame_str = os.path.splitext(fp.split('_')[-1])[0]
                    frame = int(frame_str)
                    new_meta=[]
                    for metadata in grouped_by_frame[frame]:
                        new_obj = {
                            'frame': 0,
                            'type': metadata['meta'],
                            **metadata['attributes']
                        }
                        if mode == 'state':
                            new_obj.update({'media_ids':[media_id]})
                        else:
                            new_obj.update({'media_id':media_id,
                                            'x': metadata['x'],
                                            'y': metadata['y'],
                                            'width': metadata['width'],
                                            'height': metadata['height']})
                        new_meta.append(new_obj)
                        print(f"{frame} {new_meta}")
                    ep(project, new_meta)



def _extract_thumbnails(image, localizations, outputDir):
    for localization in localizations:
        output_name = f"{localization['id']}.png"
        output_fp = os.path.join(outputDir, output_name)

        # Now get the image data from the image
        height = image.shape[0]
        width = image.shape[1]

        # Check for annotations starting off screen
        if localization.get('x') < 0:
            localization['x'] = 0
        if localization.get('y') < 0:
            localization['y'] = 0

        thumb_height = int(height * localization['height'])
        thumb_width = int(width * localization['width'])
        thumb_y = int(height * localization['y'])
        thumb_x = int(width * localization['x'])

        # Check for annotations extending off screen
        if thumb_x + thumb_width > width:
            thumb_width = width - thumb_x
        if thumb_y + thumb_height > height:
            thumb_height = height - thumb_y

        thumbnail = image[thumb_y:thumb_y+thumb_height,
                          thumb_x:thumb_x+thumb_width,
                          :]
        cv2.imwrite(output_fp, thumbnail)



import tempfile
import os

import tator

from._common import assert_vector_equal

def test_get_file(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    with tempfile.TemporaryDirectory() as temp_dir:
        outpath = os.path.join(temp_dir, "video.mp4")
        for progress in tator.download_media(tator_api, video_obj, outpath):
            print(f"Video download progress: {progress}%")
        assert(os.path.exists(outpath))

def test_get_audio(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    audio = video_obj.media_files.audio
    assert len(audio) > 0
    assert audio[0].codec == 'aac'

def test_get_by_id(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)
    other_obj = tator_api.get_media_list_by_id(project, [video])[0]
    assert video_obj.id == other_obj.id



#!/usr/bin/python3

import argparse
import json
import sys
import subprocess

FRAGMENT_VERSION=2

def parse_args():
    parser=argparse.ArgumentParser()
    parser.add_argument("input", help="MP4 File", type=str)
    parser.add_argument("-o", "--output")
    return parser.parse_args()

def make_fragment_info(video_file, output):
    cmd=["mp4dump",
         "--format",
         "json",
         video_file]
    proc=subprocess.Popen(cmd, stdout=subprocess.PIPE)
    mp4_data,error=proc.communicate()

    cmd=["ffprobe",
         "-v",
         "error",
         "-show_entries",
         "stream",
         "-print_format",
         "json",
         "-select_streams", "v",
         video_file]
    proc=subprocess.Popen(cmd, stdout=subprocess.PIPE)
    ffprobe_output,error=proc.communicate()

    ffprobe_data=json.loads(ffprobe_output)
    start_time = 0
    try:
        for stream in ffprobe_data["streams"]:
            if stream["codec_type"] == "video":
                start_time=float(ffprobe_data["streams"][0]["start_time"])
                break
    except Exception as e:
        print(e)

    outputFile=sys.stdout
    if output != None:
        outputFile=open(output, "w")

    currentOffset=0
    currentFrame=0
    info={"file": {"start": start_time, "version": FRAGMENT_VERSION}, "segments" : []}
    with open(video_file) as fp:
        obj=json.loads(mp4_data)
        for data in obj:
            block={"name": data['name'],
                   "offset": currentOffset,
                   "size": data['size']}

            # Add time offset for moof blocks
            if block['name'] == 'moof':
                for child in data['children']:
                    if child['name'] == 'traf':
                        for grandchild in child['children']:
                            if grandchild['name'] == 'trun':
                                block['frame_start'] = currentFrame
                                block['frame_samples'] = grandchild['sample count']
                                currentFrame += grandchild['sample count']
            info['segments'].append(block)
            
            currentOffset+=block['size']


    json.dump(info, outputFile)
            
if __name__=="__main__":
    args = parse_args()
    make_fragment_info(args['input'], args['output'])



import tator

def test_clone_membership_same_host(host, token, project, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator_api.whoami()
    user_id = response.id
    response = tator_api.create_membership(project, membership_spec={'user': user_id,
                                                                     'permission': 'Full Control'})
    membership = response.id
    response = tator.util.clone_membership(tator_api, membership, clone_project)
    assert(isinstance(response, tator.models.CreateResponse))

def test_clone_membership_different_host(host, token, project, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator_api.whoami()
    user_id = response.id
    response = tator_api.create_membership(project, membership_spec={'user': user_id,
                                                                     'permission': 'Full Control'})
    membership = response.id
    response = tator.util.clone_membership(tator_api, membership, clone_project, tator_api)
    assert(isinstance(response, tator.models.CreateResponse))




# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class SectionUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'annotation_bools': 'list[dict(str, object)]',
        'lucene_string': 'str',
        'media_bools': 'list[dict(str, object)]',
        'name': 'str',
        'tator_user_sections': 'str',
        'visible': 'bool'
    }

    attribute_map = {
        'annotation_bools': 'annotation_bools',
        'lucene_string': 'lucene_string',
        'media_bools': 'media_bools',
        'name': 'name',
        'tator_user_sections': 'tator_user_sections',
        'visible': 'visible'
    }

    def __init__(self, annotation_bools=None, lucene_string=None, media_bools=None, name=None, tator_user_sections=None, visible=None, local_vars_configuration=None):  # noqa: E501
        """SectionUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._annotation_bools = None
        self._lucene_string = None
        self._media_bools = None
        self._name = None
        self._tator_user_sections = None
        self._visible = None
        self.discriminator = None

        if annotation_bools is not None:
            self.annotation_bools = annotation_bools
        if lucene_string is not None:
            self.lucene_string = lucene_string
        if media_bools is not None:
            self.media_bools = media_bools
        if name is not None:
            self.name = name
        if tator_user_sections is not None:
            self.tator_user_sections = tator_user_sections
        if visible is not None:
            self.visible = visible

    @property
    def annotation_bools(self):
        """
        List of elasticsearch boolean queries that should be applied to annotations. These are applied to the boolean query \"filter\" list.

        :return: The annotation_bools of this SectionUpdate. 
        :rtype: list[dict(str, object)]
        """
        return self._annotation_bools

    @annotation_bools.setter
    def annotation_bools(self, annotation_bools):
        """
        List of elasticsearch boolean queries that should be applied to annotations. These are applied to the boolean query \"filter\" list.

        :param annotation_bools: The annotation_bools of this SectionUpdate.
        :type: list[dict(str, object)]
        """

        self._annotation_bools = annotation_bools

    @property
    def lucene_string(self):
        """
        Lucene query syntax search string.

        :return: The lucene_string of this SectionUpdate. 
        :rtype: str
        """
        return self._lucene_string

    @lucene_string.setter
    def lucene_string(self, lucene_string):
        """
        Lucene query syntax search string.

        :param lucene_string: The lucene_string of this SectionUpdate.
        :type: str
        """

        self._lucene_string = lucene_string

    @property
    def media_bools(self):
        """
        List of elasticsearch boolean queries that should be applied to media. These are applied to the boolean query \"filter\" list.

        :return: The media_bools of this SectionUpdate. 
        :rtype: list[dict(str, object)]
        """
        return self._media_bools

    @media_bools.setter
    def media_bools(self, media_bools):
        """
        List of elasticsearch boolean queries that should be applied to media. These are applied to the boolean query \"filter\" list.

        :param media_bools: The media_bools of this SectionUpdate.
        :type: list[dict(str, object)]
        """

        self._media_bools = media_bools

    @property
    def name(self):
        """
        Unique name of the algorithm workflow.

        :return: The name of this SectionUpdate. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Unique name of the algorithm workflow.

        :param name: The name of this SectionUpdate.
        :type: str
        """

        self._name = name

    @property
    def tator_user_sections(self):
        """
        Attribute that is applied to media to identify membership to a section.

        :return: The tator_user_sections of this SectionUpdate. 
        :rtype: str
        """
        return self._tator_user_sections

    @tator_user_sections.setter
    def tator_user_sections(self, tator_user_sections):
        """
        Attribute that is applied to media to identify membership to a section.

        :param tator_user_sections: The tator_user_sections of this SectionUpdate.
        :type: str
        """

        self._tator_user_sections = tator_user_sections

    @property
    def visible(self):
        """
        Determines the visibility in the UI.

        :return: The visible of this SectionUpdate. 
        :rtype: bool
        """
        return self._visible

    @visible.setter
    def visible(self, visible):
        """
        Determines the visibility in the UI.

        :param visible: The visible of this SectionUpdate.
        :type: bool
        """

        self._visible = visible

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SectionUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SectionUpdate):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Version(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'bases': 'list[int]',
        'created_by': 'str',
        'description': 'str',
        'id': 'int',
        'name': 'str',
        'number': 'int',
        'project': 'int',
        'show_empty': 'bool'
    }

    attribute_map = {
        'bases': 'bases',
        'created_by': 'created_by',
        'description': 'description',
        'id': 'id',
        'name': 'name',
        'number': 'number',
        'project': 'project',
        'show_empty': 'show_empty'
    }

    def __init__(self, bases=None, created_by=None, description='', id=None, name=None, number=None, project=None, show_empty=True, local_vars_configuration=None):  # noqa: E501
        """Version - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._bases = None
        self._created_by = None
        self._description = None
        self._id = None
        self._name = None
        self._number = None
        self._project = None
        self._show_empty = None
        self.discriminator = None

        if bases is not None:
            self.bases = bases
        if created_by is not None:
            self.created_by = created_by
        if description is not None:
            self.description = description
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if number is not None:
            self.number = number
        if project is not None:
            self.project = project
        if show_empty is not None:
            self.show_empty = show_empty

    @property
    def bases(self):
        """
        Array of other version IDs that are dependencies of this version.

        :return: The bases of this Version. 
        :rtype: list[int]
        """
        return self._bases

    @bases.setter
    def bases(self, bases):
        """
        Array of other version IDs that are dependencies of this version.

        :param bases: The bases of this Version.
        :type: list[int]
        """

        self._bases = bases

    @property
    def created_by(self):
        """
        Name of user who created the last unmodified annotation in this version.

        :return: The created_by of this Version. 
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """
        Name of user who created the last unmodified annotation in this version.

        :param created_by: The created_by of this Version.
        :type: str
        """

        self._created_by = created_by

    @property
    def description(self):
        """
        Description of the version.

        :return: The description of this Version. 
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Description of the version.

        :param description: The description of this Version.
        :type: str
        """

        self._description = description

    @property
    def id(self):
        """
        Unique integer identifying a membership.

        :return: The id of this Version. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying a membership.

        :param id: The id of this Version.
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """
        Name of the version.

        :return: The name of this Version. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the version.

        :param name: The name of this Version.
        :type: str
        """

        self._name = name

    @property
    def number(self):
        """
        Version number.

        :return: The number of this Version. 
        :rtype: int
        """
        return self._number

    @number.setter
    def number(self, number):
        """
        Version number.

        :param number: The number of this Version.
        :type: int
        """

        self._number = number

    @property
    def project(self):
        """
        Unique integer identifying a project.

        :return: The project of this Version. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying a project.

        :param project: The project of this Version.
        :type: int
        """

        self._project = project

    @property
    def show_empty(self):
        """
        Whether to show this version on media for which no annotations exist.

        :return: The show_empty of this Version. 
        :rtype: bool
        """
        return self._show_empty

    @show_empty.setter
    def show_empty(self, show_empty):
        """
        Whether to show this version on media for which no annotations exist.

        :param show_empty: The show_empty of this Version.
        :type: bool
        """

        self._show_empty = show_empty

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Version):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Version):
            return True

        return self.to_dict() != other.to_dict()



import math
import os
import requests
import subprocess
import tempfile
import logging
from uuid import uuid1

from PIL import Image

from .md5sum import md5sum
from ._upload_file import _upload_file
from ._download_file import _download_file
from ..openapi.tator_openapi.models import MessageResponse

logger = logging.getLogger(__name__)

def make_multi_stream(api, type_id, layout, name, media_ids, section, quality=None):
    """ Uploads a single media file.

    :param api: :class:`tator.TatorApi` object.
    :param type_id: Unique integer identifying a multi-stream media type.
    :param layout: 2 element list of integers defining layout as [rows, cols].
    :param name: Name of the file to use
    :param media_ids: List of media_ids to multi-stream
    :param section: Section name. If this section does not exist it will be created.
    :param quality: [Optional] Media section to upload to.
    :returns: Response from media object creation.
    """

    # Fetch the stuff we need to download files
    config = api.api_client.configuration
    host = config.host
    token = config.api_key['Authorization']
    prefix = config.api_key_prefix['Authorization']

    # use a multi extension
    name += ".multi"

    # Fetch the media type
    multi_stream_type = api.get_media_type(type_id)
    project = multi_stream_type.project
   
    # Fetch the section. If it does not exist, create it. 
    sections = api.get_section_list(project, name=section)
    if len(sections) == 0:
        section_spec = {'name': section, 'tator_user_sections': str(uuid1())}
        response = api.create_section(project, section_spec=section_spec)
        section_obj = api.get_section_list(project, name=section)[0]
    else:
        section_obj = sections[0]

    assert(len(media_ids) == layout[0]*layout[1])

    media_objects = api.get_media_list(project, media_id=media_ids)
    assert(len(media_objects) == len(media_ids))

    media_lookup={}
    for media in media_objects:
        media_lookup[media.id] = media

    attributes={}
    if section:
        attributes.update({"tator_user_sections": section_obj.tator_user_sections})

    headers = {
        'Authorization': f'{prefix} {token}',
        'Content-Type': f'application/json',
        'Accept-Encoding': 'gzip',
    }

    # Download the thumbnails into a temporary
    with tempfile.TemporaryDirectory() as d:
        for pos, media_id in enumerate(media_ids):
            media = media_lookup[media_id]
            thumbnails = media.media_files.thumbnail
            if thumbnails:
                thumb = thumbnails[0].path
            else:
                thumb = "/media/" + media.thumbnail
            thumbnail_gifs = media.media_files.thumbnail_gif
            if thumbnail_gifs:
                thumb_gif = thumbnail_gifs[0].path
            else:
                thumb_gif = "/media/" + media.thumbnail_gif
            for _ in _download_file(api, media.project, thumb,
                                    os.path.join(d, f"thumb_{pos:09d}.jpg")):
                pass
            for _ in _download_file(api, media.project, thumb_gif,
                                    os.path.join(d, f"gif_{pos:09d}.gif")):
                pass

        cmd = ["ffmpeg",
               "-y",
               "-i", "thumb_%09d.jpg",
               "-vf",f"tile={layout[1]}x{layout[0]},scale=256:-1",
               os.path.join(d,"tiled_thumb.jpg")]
        subprocess.run(cmd,cwd=d,check=True)

        input_files=[]
        rows=layout[0]
        cols=layout[1]
        filter_graph=""
        idx = 0
        for row in range(rows):
            for col in range(cols):
                filter_graph += f"[{idx}:v]"
                idx+=1
                if cols > 1 and col + 1 == cols:
                    filter_graph += f"hstack=inputs={cols}[r{row}];"
        for row in range(rows):
            if cols > 1:
                filter_graph += f"[r{row}]"
        if rows > 1 and row + 1 == rows:
            filter_graph+=f'vstack=inputs={rows}[tiled_gif];[tiled_gif]'
        filter_graph+=f'scale=256:-1[raw];[raw]split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse[final]'
        print(filter_graph)
        for pos,_ in enumerate(media_ids):
            input_files.extend(['-i', f'gif_{pos:09d}.gif'])
        cmd = ["ffmpeg",
               "-y",
               *input_files,
               "-filter_complex", filter_graph,
               "-map", "[final]",
               "-shortest",
               "tiled_gif.gif"]

        subprocess.run(cmd, cwd=d, check=True)

        md5=md5sum(os.path.join(d,'tiled_gif.gif'))

        media_spec = {'attributes': attributes,
                      'name': name,
                      'md5': md5,
                      'section': section_obj.name,
                      'type': type_id}

        resp = api.create_media(project, media_spec)
        print(f"Created {resp.id}")

        thumb_path = os.path.join(d,'tiled_thumb.jpg')
        thumb_gif_path = os.path.join(d,'tiled_gif.gif')
        for progress, thumbnail_info in _upload_file(api, project, thumb_path,
                                                     media_id=resp.id, filename='tiled_thumb.jpg'):
            logger.info(f"Thumbnail upload progress: {progress}%")
        for progress, thumbnail_gif_info in _upload_file(api, project, thumb_gif_path,
                                                         media_id=resp.id, filename='tiled_gif.gif'):
            logger.info(f"Thumbnail gif upload progress: {progress}%")

        # Open images to get output resolution.
        thumb_image = Image.open(thumb_path)
        thumb_gif_image = Image.open(thumb_gif_path)

        # Create image definitions for thumbnails.
        thumb_def = {'path': thumbnail_info.key,
                     'size': os.stat(thumb_path).st_size,
                     'resolution': [thumb_image.height, thumb_image.width],
                     'mime': f'image/{thumb_image.format.lower()}'}
        thumb_gif_def = {'path': thumbnail_gif_info.key,
                         'size': os.stat(thumb_gif_path).st_size,
                         'resolution': [thumb_gif_image.height, thumb_gif_image.width],
                         'mime': f'image/{thumb_gif_image.format.lower()}'}

        response = api.create_image_file(resp.id, role='thumbnail', image_definition=thumb_def)
        assert isinstance(response, MessageResponse)
        response = api.create_image_file(resp.id, role='thumbnail_gif', image_definition=thumb_gif_def)
        assert isinstance(response, MessageResponse)

        # Add the multi definition.
        multi_def = {"layout": layout,
                     "ids": media_ids}
        if quality:
            multi_def.update({"quality": quality})
        api.update_media(resp.id, {"multi": multi_def})

        return resp




#!/usr/bin/env python

import argparse
import os
from uuid import uuid1
import logging

from progressbar import progressbar

from ..util import md5sum

from .create_media import create_media
from .determine_transcode import determine_transcode
from .transcode import convert_streaming
from .transcode import convert_archival
from .transcode import convert_audio
from .delete_media import delete_media
from .make_thumbnails import make_thumbnails

logger = logging.getLogger(__name__)

def parse_args():
    parser = argparse.ArgumentParser(description='Full transcode pipeline on a directory of files.')
    parser.add_argument('path', type=str, help='Path to directory containing video files, or a video file.')
    parser.add_argument('--extension', type=str, default='mp4', help='File extension to upload. '
                                                                     'Ignored if path is a file.')
    parser.add_argument('--host', type=str, default='https://www.tatorapp.com', help='Host URL.')
    parser.add_argument('--token', type=str, help='REST API token.')
    parser.add_argument('--project', type=int, help='Unique integer specifying project ID.')
    parser.add_argument('--type', type=int, help='Unique integer specifying a media type.')
    parser.add_argument('--section', type=str, help='Media section name.')
    parser.add_argument('--work_dir', type=str, help='Path to working directory. If not given, '
                                                     'the path containing videos will be used.')
    return parser.parse_args()

def get_file_paths(path, base):
    paths = {
        'original': path,
        'transcoded': base + '_transcoded',
        'thumbnail': base + '_thumbnail.jpg',
        'thumbnail_gif': base + '_thumbnail_gif.gif',
        'segments': base + '_segments.json',
    }
    return paths

def transcode_single(path, args, gid):
    """Transcodes a single file.
    """
    # Get file paths.
    if args.work_dir:
        base = os.path.join(args.work_dir, os.path.splitext(os.path.basename(path))[0])
    else:
        base, _ = os.path.splitext(path)
    paths = get_file_paths(path, base)

    # Get md5 for the file.
    md5 = md5sum(paths['original'])

    # Get base filename.
    name = os.path.basename(paths['original'])

    # Generate a UID for this file.
    uid = str(uuid1())

    # Create the media object.
    media_id = create_media(args.host, args.token, args.project, args.type, args.section, name, md5,
                            gid, uid)

    try:
        # Make thumbnails.
        make_thumbnails(args.host, args.token, media_id, paths['original'], paths['thumbnail'],
                        paths['thumbnail_gif'])

        # Determine transcodes that need to be done.
        workloads = determine_transcode(args.host, args.token, args.type, path, group_to=1080)

        # Transcode the video file.
        for workload in workloads:
            category = workload['category']
            del workload['category']
            del workload['id']
            if category == 'streaming':
                convert_streaming(**workload, host=args.host, token=args.token, media=media_id,
                                  outpath=paths['transcoded'])
            elif category == 'archival':
                del workload['configs']
                convert_archival(**workload, host=args.host, token=args.token, media=media_id,
                                 outpath=paths['transcoded'])
            elif category == 'audio':
                del workload['configs']
                del workload['raw_width']
                del workload['raw_height']
                convert_audio(**workload, host=args.host, token=args.token, media=media_id,
                              outpath=paths['transcoded'])
    except:
        logging.exception('')
        delete_media(args.host, args.token, media_id)
        raise RuntimeError(f"Transcode of file {path} failed!")
    
if __name__ == '__main__':
    args = parse_args()
    gid = str(uuid1())
    if os.path.isdir(args.path):
        file_list = []
        for root, dirs, files in os.walk(args.path):
            for fname in files:
                if fname.endswith(args.extension):
                    path = os.path.join(root, fname)
                    file_list.append(path)
        for path in progressbar(file_list):
            transcode_single(path, args, gid)
    else:
        transcode_single(args.path, args, gid)



import tator

def test_analysis(
        host: str,
        token: str,
        project: int) -> None:
    """
    Create a set of analyses objects using the POST operation
    Retrieve the set of analyses objects
    Update the analysis object
    Get the object and make sure it was updated
    Delete the object
    Retrieve the set of analyses objects and make sure that object was deleted
    """

    # Setup the interface to tator
    tator_api = tator.get_api(host=host, token=token)

    # Create the analysis records
    name = "TestAnalysis"
    num_of_analyses = 10
    existing_num_of_analyses = len(tator_api.get_analysis_list(project=project))

    for count in range(num_of_analyses):
        data_query = f'random_query:{count}'
        spec = tator.models.AnalysisSpec(name=name, data_query=data_query)
        _ = tator_api.create_analysis(project=project, analysis_spec=spec)
        
    # Retrieve the analyses and verify the count matches
    analyses = tator_api.get_analysis_list(project=project)
    assert len(analyses) == existing_num_of_analyses + num_of_analyses

    # Update the analysis object
    obj = analyses[1]
    test_obj_id = obj.id
    spec = tator.models.Analysis(name="TestAnalysis2", data_query="another_random_query:0")
    _ = tator_api.update_analysis(id=test_obj_id, analysis_spec=spec)
    response = tator_api.get_analysis(id=test_obj_id)
    spec.id = test_obj_id
    assert response == spec

    # Remove the analysis object
    _ = tator_api.delete_analysis(id=test_obj_id)
    analyses = tator_api.get_analysis_list(project=project)
    analysis_is_gone = True
    for obj in analyses:
        if obj.id == test_obj_id:
            analysis_is_gone = False
            break 

    assert analysis_is_gone
    assert len(analyses) == existing_num_of_analyses + num_of_analyses - 1



import tator
import datetime

from ._common import assert_vector_equal

def _make_box(project, box_type, video, frame):
    return {
        'x': 0,
        'y': 0,
        'width': 1,
        'height': 1,
        'project': project,
        'type': box_type,
        'media_id': video,
        'frame': frame,
    }

def test_stategraphic(host, token, project, video, box_type, track_type):
    TRACK_LENGTH=200

    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    # Make boxes for track.
    boxes = [_make_box(project, box_type, video, frame) for frame in range(TRACK_LENGTH)]
    response = tator_api.create_localization_list(project, localization_spec=boxes)
    assert isinstance(response, tator.models.CreateListResponse)
    print(response.message)
    box_ids = response.id

    # Make track.
    response = tator_api.create_state_list(project, state_spec=[{
        'project': project,
        'type': track_type,
        'media_ids': [video],
        'localization_ids': box_ids,
    }])
    print(response.message)
    assert isinstance(response, tator.models.CreateListResponse)
    track_id = response.id[0]

    # Get state graphic.
    t0 = datetime.datetime.now()
    stategraphic = tator.util.full_state_graphic(tator_api, track_id)
    t1 = datetime.datetime.now()
    print(f"State graphic took {t1 - t0} seconds.")
    assert(len(stategraphic) == TRACK_LENGTH)
    for frame_data in stategraphic:
        size = (frame_data.height, frame_data.width, len(frame_data.mode))
        assert_vector_equal(size, (224,224,3))




from uuid import uuid1
import mimetypes
import os
import math
import io
import tarfile
import tempfile

from urllib.parse import urljoin
from urllib.parse import urlsplit

from ._upload_file import _upload_file
from .md5sum import md5sum

def upload_media_archive(api, project, paths, section="Test Section", chunk_size=10*1024*1024):
    """ Uploads multiple media files as an archive.

    Example:

    .. code-block:: python

        api = tator.get_api(host, token)
        for progress, response in tator.util.upload_media_archive(api, project_id, paths):
            print(f"Upload progress: {progress}%")
        print(response.message)

    :param api: :class:`tator.TatorApi` object.
    :param project: Unique integer identifying a project.
    :param paths: Path to a media archive or list of paths to media files.
    :param section: [Optional] Media section to upload to.
    :param chunk_size: [Optional] Chunk size in bytes. Default is 2MB.
    :returns: Generator that yields tuple containing progress (0-100) and a
        response. The response is `None` until the last yield, when the response
        is the response object from :meth:`tator.TatorApi.transcode`.
    """
    upload_uid = str(uuid1())
    upload_gid = str(uuid1())
    if isinstance(paths, list):
        fobj = tempfile.NamedTemporaryFile(delete=False)    
        in_mem_tar = tarfile.TarFile(mode='w', fileobj=fobj)
        for idx, fp in enumerate(paths):
            in_mem_tar.add(fp, os.path.basename(fp))
        fobj.close()
        path = fobj.name
    else:
        path = paths
    for progress, upload_info in _upload_file(api, project, path):
        yield (progress, None)
    url = api.get_download_info(project, {'keys': [upload_info.key]})[0].url

    # Initiate transcode.
    spec = {
        'type': -1, #Tar-based import
        'uid': upload_uid,
        'gid': upload_gid,
        'url': url,
        'name': "archive.tar",
        'section': section,
        'md5': "N/A",
    }
    response = api.transcode(project, transcode_spec=spec)
    yield (100, response)



import pytest
from uuid import uuid4

import tator

allowed_mutations = {
    "bool": ["bool", "enum", "string"],
    "int": ["int", "float", "enum", "string"],
    "float": ["int", "float", "enum", "string"],
    "enum": ["enum", "string"],
    "string": ["enum", "string"],
    "datetime": ["enum", "string", "datetime"],
    "geopos": ["enum", "string", "geopos"],
}


def mutation_helper(tator_api, type_getter, type_id, params):
    uid = f" {uuid4()}"
    source_name = params["source_name"]
    dest_name = params.get("dest_name", source_name)
    source_name += uid
    dest_name += uid
    source_dtype = params["source_dtype"]
    dest_dtype = params.get("dest_dtype", source_dtype)
    assert_cnt = 0

    entity_type = type_getter(type_id)
    addition = {
        "entity_type": f"{type(entity_type).__name__}",
        "addition": {"name": source_name, "dtype": source_dtype},
    }
    if source_dtype == "enum":
        addition["addition"]["choices"] = ["a", "b", "c"]
    tator_api.add_attribute(id=type_id, attribute_type_spec=addition)
    entity_type = type_getter(type_id)

    # Check attribute name before changing it
    assert any(attr.name == source_name for attr in entity_type.attribute_types)
    assert_cnt += 1

    for attr in entity_type.attribute_types:
        if attr.name == source_name:
            # Check the `dtype` before changing it
            assert attr.dtype == source_dtype
            assert_cnt += 1
            break
    else:
        # If the new attribute is not found above, we should fail
        assert False

    mutation = {
        "global": "false",
        "entity_type": addition["entity_type"],
        "old_attribute_type_name": source_name,
        "new_attribute_type": {"name": dest_name, "dtype": dest_dtype},
    }
    if dest_dtype == "enum":
        mutation["new_attribute_type"]["choices"] = ["a", "b", "c"]

    # A type mutation not in the list of allowed mutations will raise an `ApiException` and the
    # expected `dtype` will be the same as `source_dtype`
    if dest_dtype in allowed_mutations[source_dtype]:
        expected_dtype = dest_dtype
        expected_name = dest_name
        tator_api.rename_attribute(id=type_id, attribute_type_update=mutation)
    else:
        expected_dtype = source_dtype
        expected_name = source_name
        with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException) as excinfo:
            tator_api.rename_attribute(id=type_id, attribute_type_update=mutation)

    entity_type = type_getter(type_id)

    # Check new attribute name
    assert any(attr.name == expected_name for attr in entity_type.attribute_types)
    assert_cnt += 1

    # No attributes should have the old name, if we are expecting a change
    if source_name != expected_name:
        assert all(attr.name != source_name for attr in entity_type.attribute_types)
        assert_cnt += 1

    for attr in entity_type.attribute_types:
        if attr.name == expected_name:
            # Check that the `dtype` has been updated
            assert attr.dtype == expected_dtype
            assert_cnt += 1
            break
    else:
        # If the new attribute is not found above, we should fail
        assert False

    assert assert_cnt == params["expected_asserts"]


@pytest.mark.parametrize("source_dtype", allowed_mutations.keys())
@pytest.mark.parametrize("dest_dtype", allowed_mutations.keys())
def test_box_type_dtype_change(host, token, project, attribute_box_type, source_dtype, dest_dtype):
    tator_api = tator.get_api(host, token)
    params = {
        "source_name": f"{source_dtype} attribute to modify",
        "source_dtype": source_dtype,
        "dest_dtype": dest_dtype,
        "expected_asserts": 4,
    }
    mutation_helper(tator_api, tator_api.get_localization_type, attribute_box_type, params)


@pytest.mark.parametrize("dtype", allowed_mutations.keys())
def test_box_type_invalid_dtype_change(host, token, project, attribute_box_type, dtype):
    tator_api = tator.get_api(host, token)
    params = {
        "source_name": f"{dtype} attribute to modify",
        "source_dtype": dtype,
        "dest_dtype": "unknown",
        "expected_asserts": 4,
    }
    mutation_helper(tator_api, tator_api.get_localization_type, attribute_box_type, params)


@pytest.mark.parametrize("dtype", allowed_mutations.keys())
def test_box_type_name_change(host, token, project, attribute_box_type, dtype):
    tator_api = tator.get_api(host, token)
    params = {
        "source_name": f"{dtype} attribute to rename",
        "source_dtype": dtype,
        "dest_name": f"renamed {dtype} attribute",
        "expected_asserts": 5,
    }
    mutation_helper(tator_api, tator_api.get_localization_type, attribute_box_type, params)


@pytest.mark.parametrize("source_dtype", allowed_mutations.keys())
@pytest.mark.parametrize("dest_dtype", allowed_mutations.keys())
def test_box_type_full_mutation(host, token, project, attribute_box_type, source_dtype, dest_dtype):
    params = {
        "source_name": f"{source_dtype} attribute to modify",
        "source_dtype": source_dtype,
        "dest_name": f"renamed {source_dtype} -> {dest_dtype} attribute",
        "dest_dtype": dest_dtype,
        "expected_asserts": 5 if dest_dtype in allowed_mutations[source_dtype] else 4,
    }
    tator_api = tator.get_api(host, token)
    mutation_helper(tator_api, tator_api.get_localization_type, attribute_box_type, params)


@pytest.mark.parametrize("dtype", allowed_mutations.keys())
def test_video_and_image_type_name_change(host, token, project, attribute_video_type, image_type, dtype):
    uid = f"{uuid4()}"
    source_name = f"{dtype} attribute to rename {uid}"
    dest_name = f"renamed {dtype} attribute {uid}"
    source_dtype = dtype
    tator_api = tator.get_api(host, token)

    # Add attribute to attribute_video_type
    entity_type = tator_api.get_media_type(attribute_video_type)
    addition = {
        "entity_type": "MediaType",
        "addition": {"name": source_name, "dtype": source_dtype},
    }
    if source_dtype == "enum":
        addition["addition"]["choices"] = ["a", "b", "c"]
    tator_api.add_attribute(id=attribute_video_type, attribute_type_spec=addition)
    entity_type = tator_api.get_media_type(attribute_video_type)

    # Check attribute name before changing it
    assert any(attr.name == source_name for attr in entity_type.attribute_types)

    # Add same attribute to image_type
    entity_type = tator_api.get_media_type(image_type)
    addition = {
        "entity_type": "MediaType",
        "addition": {"name": source_name, "dtype": source_dtype},
    }
    if source_dtype == "enum":
        addition["addition"]["choices"] = ["a", "b", "c"]
    tator_api.add_attribute(id=image_type, attribute_type_spec=addition)
    entity_type = tator_api.get_media_type(image_type)

    # Check attribute name before changing it
    assert any(attr.name == source_name for attr in entity_type.attribute_types)

    # Mutate attribute on attribute_video_type
    mutation = {
        "global": "true",
        "entity_type": "MediaType",
        "old_attribute_type_name": source_name,
        "new_attribute_type": {"name": dest_name, "dtype": source_dtype},
    }
    if source_dtype == "enum":
        mutation["new_attribute_type"]["choices"] = ["a", "b", "c"]

    tator_api.rename_attribute(id=attribute_video_type, attribute_type_update=mutation)

    entity_type = tator_api.get_media_type(attribute_video_type)

    # Check new attribute name
    assert any(attr.name == dest_name for attr in entity_type.attribute_types)

    # No attributes should have the old name
    assert all(attr.name != source_name for attr in entity_type.attribute_types)

    # The image_type attribute should also have the new name
    entity_type = tator_api.get_media_type(image_type)
    assert any(attr.name == dest_name for attr in entity_type.attribute_types)
    assert all(attr.name != source_name for attr in entity_type.attribute_types)



import tator
import time
from uuid import uuid1

def test_transcode_fault_handling(host, token, project):
    """ Verify Error Handling occurs for transcode endpoint """
    tator_api = tator.get_api(host, token)
    exception = False
    try:
        video_obj = tator_api.transcode(project,
                                        transcode_spec={'gid':str(uuid1()),
                                                        'md5':"1234",
                                                        'name':"blah",
                                                        'section':"blah",
                                                        'type':200,
                                                        'uid':str(uuid1()),
                                                        'url':"blah"})
    except Exception as e:
        exception = True
        code = e.status

    assert(exception)
    assert(code == 400)

def test_transcode_existing_media(host, token, project, video_type, video_file):
    tator_api = tator.get_api(host, token)

    # Compute parameters needed to create media.
    md5 = tator.util.md5sum(video_file)
    upload_uid = str(uuid1())
    upload_gid = str(uuid1())
    fname='ExistingMediaTranscodeTest.mp4'
    section="Transcoded with Existing Media"

    # Define media spec.
    media_spec = {
        'type': video_type,
        'uid': upload_uid,
        'gid': upload_gid,
        'name': fname,
        'section': section,
        'md5': md5,
    }

    # Create the media.
    response = tator_api.create_media(project=project, media_spec=media_spec)
    print(f"Transcoding video with existing media ID {response.id}...")
    for progress, response in tator.util.upload_media(tator_api, video_type, video_file,
                                                      media_id=response.id):
        print(f"Upload video progress: {progress}%")
    print(response.message)
    while True:
        job = tator_api.get_job(response.uid)
        if job.status == 'Succeeded':
            break
        elif job.status == 'Failed':
            raise Exception('Transcode of existing media failed!')
        else:
            print("Waiting for transcode of existing media to complete...")
            time.sleep(2.5)




import datetime
import random
import uuid

import tator
from ._common import assert_close_enough

def random_localization(project, box_type, video_obj, post=False):
    x = random.uniform(0.0, 1.0)
    y = random.uniform(0.0, 1.0)
    w = random.uniform(0.0, 1.0 - x)
    h = random.uniform(0.0, 1.0 - y)
    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
    }
    out = {
        'x': x,
        'y': y,
        'width': w,
        'height': h,
        'project': project,
        'type': box_type,
        'media_id': video_obj.id,
        'frame': random.randint(0, video_obj.num_frames - 1),
    }
    if post:
        out = {**out, **attributes}
    else:
        out['attributes'] = attributes
    return out

def comparison_query(tator_api, project, exclude):
    """ Runs a random query and compares results with ES enabled and disabled.
    """
    bool_value = random.choice([True, False])
    int_lower = random.randint(-1000, 0)
    int_upper = random.randint(0, 1000)
    float_lower = random.uniform(-1000.0, 0.0)
    float_upper = random.uniform(0.0, 1000.0)
    enum_value = random.choice(['a', 'b', 'c'])
    attribute_filter = [f"test_bool::{'true' if bool_value else 'false'}", f"test_enum::{enum_value}"]
    attribute_lte_filter = [f"test_int::{int_upper}"]
    attribute_gte_filter = [f"test_int::{int_lower}"]
    attribute_lt_filter = [f"test_float::{float_upper}"]
    attribute_gt_filter = [f"test_float::{float_lower}"]
    t0 = datetime.datetime.now()
    from_psql = tator_api.get_localization_list(project,
                                                attribute=attribute_filter,
                                                attribute_lte=attribute_lte_filter,
                                                attribute_gte=attribute_gte_filter,
                                                attribute_lt=attribute_lt_filter,
                                                attribute_gt=attribute_gt_filter)
    psql_time = datetime.datetime.now() - t0
    t0 = datetime.datetime.now()
    from_es = tator_api.get_localization_list(project,
                                              attribute=attribute_filter,
                                              attribute_lte=attribute_lte_filter,
                                              attribute_gte=attribute_gte_filter,
                                              attribute_lt=attribute_lt_filter,
                                              attribute_gt=attribute_gt_filter,
                                              force_es=1)
    es_time = datetime.datetime.now() - t0
    for psql, es in zip(from_psql, from_es):
        assert_close_enough(psql, es, exclude)
        assert(psql.attributes['test_bool'] == bool_value)
        assert(psql.attributes['test_int'] <= int_upper)
        assert(psql.attributes['test_int'] >= int_lower)
        assert(psql.attributes['test_float'] < float_upper)
        assert(psql.attributes['test_float'] > float_lower)
        assert(psql.attributes['test_enum'] == enum_value)
    return psql_time, es_time

def test_localization_crud(host, token, project, video_type, video, box_type):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    # These fields will not be checked for object equivalence after patch.
    exclude = ['project', 'type', 'media_id', 'id', 'meta', 'user']

    # Test bulk create.
    num_localizations = random.randint(2000, 10000)
    boxes = [
        random_localization(project, box_type, video_obj, post=True)
        for _ in range(num_localizations)
    ]
    box_ids = []
    for response in tator.util.chunked_create(tator_api.create_localization_list,
                                         project, localization_spec=boxes):
        box_ids += response.id
    assert len(box_ids) == len(boxes)
    print(f"Created {len(box_ids)} boxes!")

    # Verify list is the right length
    response = tator_api.get_localization_list(project,type=box_type)
    assert len(response) == num_localizations

    # Test single create.
    box = random_localization(project, box_type, video_obj, post=True)
    response = tator_api.create_localization_list(project, localization_spec=[box])
    assert isinstance(response, tator.models.CreateListResponse)
    box_id = response.id[0]

    # Patch single box.
    patch = random_localization(project, box_type, video_obj)
    response = tator_api.update_localization(box_id, localization_update=patch)
    assert isinstance(response, tator.models.MessageResponse)
    print(response.message)

    # Get single box.
    updated_box = tator_api.get_localization(box_id)
    assert_close_enough(patch, updated_box, exclude)

    # Get box by ID.
    box_by_id = tator_api.get_localization_list_by_id(project, [box_id])[0]
    assert_close_enough(updated_box, box_by_id, exclude)

    # Delete single box.
    response = tator_api.delete_localization(box_id)
    assert isinstance(response, tator.models.MessageResponse)
    print(response.message)

    params = {'media_id': [video], 'type': box_type}
    assert(tator_api.get_localization_count(project, **params) == len(boxes))

    # Bulk update box attributes.
    bulk_patch = random_localization(project, box_type, video_obj)
    bulk_patch = {'attributes': bulk_patch['attributes']}
    response = tator_api.update_localization_list(project, **params,
                                                  attribute_bulk_update=bulk_patch)
    assert isinstance(response, tator.models.MessageResponse)
    print(response.message)

    # Verify all boxes have been updated.
    boxes = tator_api.get_localization_list(project, **params)
    dataframe = tator.util.to_dataframe(boxes)
    assert(len(boxes)==len(dataframe))
    for box in boxes:
        assert_close_enough(bulk_patch, box, exclude)

    # Do random queries using psql and elasticsearch and compare results.
    es_time = datetime.timedelta(seconds=0)
    psql_time = datetime.timedelta(seconds=0)
    NUM_QUERIES = 10
    for _ in range(NUM_QUERIES):
        psql, es = comparison_query(tator_api, project, exclude)
        psql_time += psql
        es_time += es
    print(f"Over {NUM_QUERIES} random attribute queries:")
    print(f"  Avg PSQL time: {psql_time / NUM_QUERIES}")
    print(f"  Avg ES time: {es_time / NUM_QUERIES}")

    # Clone boxes to same media.
    version_mapping = {version.id: version.id for version in tator_api.get_version_list(project)}
    generator = tator.util.clone_localization_list(tator_api, {**params, 'project': project},
                                                   project, version_mapping, {video:video},
                                                   {box_type: box_type}, tator_api)
    for num_created, num_total, response, id_map in generator:
        print(f"Created {num_created} of {num_total} localizations...")
    print(f"Finished creating {num_created} localizations!")
    assert(tator_api.get_localization_count(project, **params) == 2 * len(boxes))

    # Delete all boxes.
    response = tator_api.delete_localization_list(project, **params)
    assert isinstance(response, tator.models.MessageResponse)

    # Verify all boxes are gone.
    boxes = tator_api.get_localization_list(project, **params)
    assert boxes == []



import tator
import time

def test_import_image(host, token, project, image_type):
    api = tator.get_api(host, token)
    url = 'https://upload.wikimedia.org/wikipedia/en/7/7d/Lenna_%28test_image%29.png'
    response = tator.util.import_media(api, image_type, url)
    assert(isinstance(response, tator.models.CreateResponse))

def test_import_video(host, token, project, video_type):
    api = tator.get_api(host, token)
    url = 'http://www.ballastmedia.com/wp-content/uploads/AudioVideoSyncTest_BallastMedia.mp4'
    response = tator.util.import_media(api, video_type, url)
    print(response.message)
    while True:
        job = api.get_job(response.uid)
        if job.status == 'Succeeded':
            break
        elif job.status == 'Failed':
            raise Exception('Media import failed!')
        else:
            print("Waiting for transcode of imported media to complete...")
            time.sleep(2.5)




# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class MediaUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'dict(str, object)',
        'codec': 'str',
        'fps': 'float',
        'height': 'int',
        'last_edit_end': 'datetime',
        'last_edit_start': 'datetime',
        'media_files': 'MediaFiles',
        'multi': 'MultiDefinition',
        'name': 'str',
        'num_frames': 'int',
        'width': 'int'
    }

    attribute_map = {
        'attributes': 'attributes',
        'codec': 'codec',
        'fps': 'fps',
        'height': 'height',
        'last_edit_end': 'last_edit_end',
        'last_edit_start': 'last_edit_start',
        'media_files': 'media_files',
        'multi': 'multi',
        'name': 'name',
        'num_frames': 'num_frames',
        'width': 'width'
    }

    def __init__(self, attributes=None, codec=None, fps=None, height=None, last_edit_end=None, last_edit_start=None, media_files=None, multi=None, name=None, num_frames=None, width=None, local_vars_configuration=None):  # noqa: E501
        """MediaUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attributes = None
        self._codec = None
        self._fps = None
        self._height = None
        self._last_edit_end = None
        self._last_edit_start = None
        self._media_files = None
        self._multi = None
        self._name = None
        self._num_frames = None
        self._width = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        if codec is not None:
            self.codec = codec
        if fps is not None:
            self.fps = fps
        if height is not None:
            self.height = height
        if last_edit_end is not None:
            self.last_edit_end = last_edit_end
        if last_edit_start is not None:
            self.last_edit_start = last_edit_start
        if media_files is not None:
            self.media_files = media_files
        if multi is not None:
            self.multi = multi
        if name is not None:
            self.name = name
        if num_frames is not None:
            self.num_frames = num_frames
        if width is not None:
            self.width = width

    @property
    def attributes(self):
        """
        Object containing attribute values.

        :return: The attributes of this MediaUpdate. 
        :rtype: dict(str, object)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Object containing attribute values.

        :param attributes: The attributes of this MediaUpdate.
        :type: dict(str, object)
        """

        self._attributes = attributes

    @property
    def codec(self):
        """
        Codec of the original video.

        :return: The codec of this MediaUpdate. 
        :rtype: str
        """
        return self._codec

    @codec.setter
    def codec(self, codec):
        """
        Codec of the original video.

        :param codec: The codec of this MediaUpdate.
        :type: str
        """

        self._codec = codec

    @property
    def fps(self):
        """
        Frame rate of the video.

        :return: The fps of this MediaUpdate. 
        :rtype: float
        """
        return self._fps

    @fps.setter
    def fps(self, fps):
        """
        Frame rate of the video.

        :param fps: The fps of this MediaUpdate.
        :type: float
        """

        self._fps = fps

    @property
    def height(self):
        """
        Pixel height of the video.

        :return: The height of this MediaUpdate. 
        :rtype: int
        """
        return self._height

    @height.setter
    def height(self, height):
        """
        Pixel height of the video.

        :param height: The height of this MediaUpdate.
        :type: int
        """

        self._height = height

    @property
    def last_edit_end(self):
        """
        Datetime of the end of the session when this media or its annotations were last edited.

        :return: The last_edit_end of this MediaUpdate. 
        :rtype: datetime
        """
        return self._last_edit_end

    @last_edit_end.setter
    def last_edit_end(self, last_edit_end):
        """
        Datetime of the end of the session when this media or its annotations were last edited.

        :param last_edit_end: The last_edit_end of this MediaUpdate.
        :type: datetime
        """

        self._last_edit_end = last_edit_end

    @property
    def last_edit_start(self):
        """
        Datetime of the start of the session when this media or its annotations were last edited.

        :return: The last_edit_start of this MediaUpdate. 
        :rtype: datetime
        """
        return self._last_edit_start

    @last_edit_start.setter
    def last_edit_start(self, last_edit_start):
        """
        Datetime of the start of the session when this media or its annotations were last edited.

        :param last_edit_start: The last_edit_start of this MediaUpdate.
        :type: datetime
        """

        self._last_edit_start = last_edit_start

    @property
    def media_files(self):
        """

        :return: The media_files of this MediaUpdate. 
        :rtype: MediaFiles
        """
        return self._media_files

    @media_files.setter
    def media_files(self, media_files):
        """

        :param media_files: The media_files of this MediaUpdate.
        :type: MediaFiles
        """

        self._media_files = media_files

    @property
    def multi(self):
        """

        :return: The multi of this MediaUpdate. 
        :rtype: MultiDefinition
        """
        return self._multi

    @multi.setter
    def multi(self, multi):
        """

        :param multi: The multi of this MediaUpdate.
        :type: MultiDefinition
        """

        self._multi = multi

    @property
    def name(self):
        """
        Name of the media.

        :return: The name of this MediaUpdate. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the media.

        :param name: The name of this MediaUpdate.
        :type: str
        """

        self._name = name

    @property
    def num_frames(self):
        """
        Number of frames in the video.

        :return: The num_frames of this MediaUpdate. 
        :rtype: int
        """
        return self._num_frames

    @num_frames.setter
    def num_frames(self, num_frames):
        """
        Number of frames in the video.

        :param num_frames: The num_frames of this MediaUpdate.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                num_frames is not None and num_frames < 0):  # noqa: E501
            raise ValueError("Invalid value for `num_frames`, must be a value greater than or equal to `0`")  # noqa: E501

        self._num_frames = num_frames

    @property
    def width(self):
        """
        Pixel width of the video.

        :return: The width of this MediaUpdate. 
        :rtype: int
        """
        return self._width

    @width.setter
    def width(self, width):
        """
        Pixel width of the video.

        :param width: The width of this MediaUpdate.
        :type: int
        """

        self._width = width

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MediaUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MediaUpdate):
            return True

        return self.to_dict() != other.to_dict()



def _convert_for_post(leaf, leaf_type_mapping, parent_mapping):
    # Swap parent IDs.
    parent_id = leaf.parent
    if parent_id:
        if parent_id in parent_mapping:
             parent_id = parent_mapping[parent_id]
        else:
            raise ValueError(f"Source parent ID {parent_id} missing from parent_mapping!")
    # Swap leaf type IDs.
    leaf_type_id = leaf.meta
    if leaf_type_id in leaf_type_mapping:
        leaf_type_id = leaf_type_mapping[leaf_type_id]
    else:
        raise ValueError(f"Source leaf_type ID {leaf_type_id} missing from "
                          "leaf_type_mapping!")
    # Fill in required fields for post.
    spec = {'name': leaf.name,
            'type': leaf_type_id,
            'parent': parent_id,
            **leaf.attributes}
    spec = {key:spec[key] for key in spec if spec[key] is not None}
    return spec

def clone_leaf_list(src_api, query_params, dest_project, parent_mapping,
                    leaf_type_mapping, dest_api=None):
    """ Clone leaf list.

    This can be used to clone leaves from one project to another or from one
    host to another.

    Example for different host:

    .. code-block:: python

        src_api = tator.get_api(host, token)
        dest_api = tator.get_api(other_host, other_token)
        query_params = {'project': 1, 'depth': 1}
        dest_project = 1
        created_ids = []
        parent_mapping = {1: 10} # Mapping of parent leaf IDs
        leaf_type_mapping = {1: 10} # Source leaf type ID -> Dest leaf type ID
        generator = clone_leaf_list(src_api, query_params, dest_project, parent_mapping,
                                    leaf_type_mapping, dest_api)
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} leafs...")
            created_ids.append(response.id)
        print(f"Finished creating {num_created} leafs!")

    Example for same host:

    .. code-block:: python

        api = tator.get_api(host, token)
        query_params = {'project': 1, 'depth': 1}
        dest_project = 1
        parent_mapping = {1: 10}
        leaf_type_mapping = {1: 10} # Source leaf type ID -> Dest leaf type ID
        created_ids = []
        generator = clone_leaf_list(api, query_params, dest_project, parent_mapping)
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} leafs...")
            created_ids += response.id
        print(f"Finished creating {num_created} leafs!")

    :param src_api: :class:`tator.TatorApi` object corresponding to source host or only
        host if this is a clone on same host.
    :param query_params: Dictionary containing query parameters for source leaf list.
    :param dest_project: Unique integer identifying destination project.
    :param parent_mapping: Dictionary mapping source parent leaf IDs to destination parent leaf
        IDs. If the source leaf list contains a parent ID for which a destination parent ID is
        not supplied, an exception is raised.
    :param leaf_type_mapping: Dictionary mapping source leaf type IDs to destination 
        leaf_type IDs. If the source leaf list contains a leaf type ID for which a
        destination leaf type ID is not supplied, an exception is raised.
    :param dest_api: :class:`tator.TatorApi` object corresponding to destination host.
    :returns: Generator containing number of leafs created, number of leafs total,
        most recent response from leaf creation operation, and mapping between
        original IDs and created IDs.
    """
    # Make sure query has a project.
    if 'project' not in query_params:
        raise Exception("Query parameters must include a project!")

    # Set the dest_api if not given.
    if dest_api is None:
        dest_api = src_api

    # Start by getting list of leafs to be cloned.
    leafs = src_api.get_leaf_list(**query_params)

    # Convert to new spec.
    spec = [_convert_for_post(leaf, leaf_type_mapping, parent_mapping)
            for leaf in leafs]

    # Create the leafs.
    created_ids = []
    total_leafs = len(spec)
    for idx in range(0, total_leafs, 500):
        response = dest_api.create_leaf_list(dest_project,
                                              leaf_spec=spec[idx:idx+500])
        created_ids += response.id
        id_map = {src.id: dest_id for src, dest_id in zip(leafs[idx:idx+500], response.id)}
        yield (len(created_ids), total_leafs, response, id_map)




# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class TranscodeSpec(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'object',
        'gid': 'str',
        'md5': 'str',
        'media_id': 'int',
        'name': 'str',
        'section': 'str',
        'size': 'int',
        'type': 'int',
        'uid': 'str',
        'url': 'str'
    }

    attribute_map = {
        'attributes': 'attributes',
        'gid': 'gid',
        'md5': 'md5',
        'media_id': 'media_id',
        'name': 'name',
        'section': 'section',
        'size': 'size',
        'type': 'type',
        'uid': 'uid',
        'url': 'url'
    }

    def __init__(self, attributes=None, gid=None, md5=None, media_id=None, name=None, section=None, size=None, type=None, uid=None, url=None, local_vars_configuration=None):  # noqa: E501
        """TranscodeSpec - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attributes = None
        self._gid = None
        self._md5 = None
        self._media_id = None
        self._name = None
        self._section = None
        self._size = None
        self._type = None
        self._uid = None
        self._url = None
        self.discriminator = None

        self.attributes = attributes
        self.gid = gid
        self.md5 = md5
        self.media_id = media_id
        self.name = name
        self.section = section
        if size is not None:
            self.size = size
        self.type = type
        self.uid = uid
        self.url = url

    @property
    def attributes(self):
        """
        Attributes to apply upon upload

        :return: The attributes of this TranscodeSpec. 
        :rtype: object
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Attributes to apply upon upload

        :param attributes: The attributes of this TranscodeSpec.
        :type: object
        """

        self._attributes = attributes

    @property
    def gid(self):
        """
        UUID generated for the job group. This value may be associated with messages generated during upload via the `Progress` endpoint, or it may be newly generated. The transcode workflow will use this value to generate progress messages.

        :return: The gid of this TranscodeSpec. 
        :rtype: str
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """
        UUID generated for the job group. This value may be associated with messages generated during upload via the `Progress` endpoint, or it may be newly generated. The transcode workflow will use this value to generate progress messages.

        :param gid: The gid of this TranscodeSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and gid is None:  # noqa: E501
            raise ValueError("Invalid value for `gid`, must not be `None`")  # noqa: E501

        self._gid = gid

    @property
    def md5(self):
        """
        MD5 sum of the media file.

        :return: The md5 of this TranscodeSpec. 
        :rtype: str
        """
        return self._md5

    @md5.setter
    def md5(self, md5):
        """
        MD5 sum of the media file.

        :param md5: The md5 of this TranscodeSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and md5 is None:  # noqa: E501
            raise ValueError("Invalid value for `md5`, must not be `None`")  # noqa: E501

        self._md5 = md5

    @property
    def media_id(self):
        """
        ID of an existing media. If given, this media will be used for the transcode operation rather than creating a new object.

        :return: The media_id of this TranscodeSpec. 
        :rtype: int
        """
        return self._media_id

    @media_id.setter
    def media_id(self, media_id):
        """
        ID of an existing media. If given, this media will be used for the transcode operation rather than creating a new object.

        :param media_id: The media_id of this TranscodeSpec.
        :type: int
        """

        self._media_id = media_id

    @property
    def name(self):
        """
        Name of the file.

        :return: The name of this TranscodeSpec. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the file.

        :param name: The name of this TranscodeSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def section(self):
        """
        Media section name to upload to.

        :return: The section of this TranscodeSpec. 
        :rtype: str
        """
        return self._section

    @section.setter
    def section(self, section):
        """
        Media section name to upload to.

        :param section: The section of this TranscodeSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and section is None:  # noqa: E501
            raise ValueError("Invalid value for `section`, must not be `None`")  # noqa: E501

        self._section = section

    @property
    def size(self):
        """
        Size of the file in bytes. This parameter is required if the supplied URL is external (not produced by `DownloadInfo` and cannot accept HEAD requests.

        :return: The size of this TranscodeSpec. 
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """
        Size of the file in bytes. This parameter is required if the supplied URL is external (not produced by `DownloadInfo` and cannot accept HEAD requests.

        :param size: The size of this TranscodeSpec.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                size is not None and size < 0):  # noqa: E501
            raise ValueError("Invalid value for `size`, must be a value greater than or equal to `0`")  # noqa: E501

        self._size = size

    @property
    def type(self):
        """
        Unique integer identifying a video type.

        :return: The type of this TranscodeSpec. 
        :rtype: int
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Unique integer identifying a video type.

        :param type: The type of this TranscodeSpec.
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    @property
    def uid(self):
        """
        UUID generated for the individual job. This value may be associated with messages generated during upload via the `Progress` endpoint, or it may be newly generated. The transcode workflow will use this value to generate progress messages.

        :return: The uid of this TranscodeSpec. 
        :rtype: str
        """
        return self._uid

    @uid.setter
    def uid(self, uid):
        """
        UUID generated for the individual job. This value may be associated with messages generated during upload via the `Progress` endpoint, or it may be newly generated. The transcode workflow will use this value to generate progress messages.

        :param uid: The uid of this TranscodeSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and uid is None:  # noqa: E501
            raise ValueError("Invalid value for `uid`, must not be `None`")  # noqa: E501

        self._uid = uid

    @property
    def url(self):
        """
        Upload URL for the raw video.

        :return: The url of this TranscodeSpec. 
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """
        Upload URL for the raw video.

        :param url: The url of this TranscodeSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and url is None:  # noqa: E501
            raise ValueError("Invalid value for `url`, must not be `None`")  # noqa: E501

        self._url = url

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TranscodeSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TranscodeSpec):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class EncodeConfig(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'crf': 'int',
        'preset': 'str',
        'tune': 'str',
        'vcodec': 'str'
    }

    attribute_map = {
        'crf': 'crf',
        'preset': 'preset',
        'tune': 'tune',
        'vcodec': 'vcodec'
    }

    def __init__(self, crf=23, preset='fast', tune='fastdecode', vcodec='hevc', local_vars_configuration=None):  # noqa: E501
        """EncodeConfig - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._crf = None
        self._preset = None
        self._tune = None
        self._vcodec = None
        self.discriminator = None

        if crf is not None:
            self.crf = crf
        if preset is not None:
            self.preset = preset
        if tune is not None:
            self.tune = tune
        if vcodec is not None:
            self.vcodec = vcodec

    @property
    def crf(self):
        """
        Constant rate factor.

        :return: The crf of this EncodeConfig. 
        :rtype: int
        """
        return self._crf

    @crf.setter
    def crf(self, crf):
        """
        Constant rate factor.

        :param crf: The crf of this EncodeConfig.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                crf is not None and crf > 51):  # noqa: E501
            raise ValueError("Invalid value for `crf`, must be a value less than or equal to `51`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                crf is not None and crf < 0):  # noqa: E501
            raise ValueError("Invalid value for `crf`, must be a value greater than or equal to `0`")  # noqa: E501

        self._crf = crf

    @property
    def preset(self):
        """
        Preset for ffmpeg encoding.

        :return: The preset of this EncodeConfig. 
        :rtype: str
        """
        return self._preset

    @preset.setter
    def preset(self, preset):
        """
        Preset for ffmpeg encoding.

        :param preset: The preset of this EncodeConfig.
        :type: str
        """
        allowed_values = ["ultrafast", "superfast", "veryfast", "faster", "fast", "medium", "slow", "slower", "veryslow"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and preset not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `preset` ({0}), must be one of {1}"  # noqa: E501
                .format(preset, allowed_values)
            )

        self._preset = preset

    @property
    def tune(self):
        """
        Tune setting for ffmpeg.

        :return: The tune of this EncodeConfig. 
        :rtype: str
        """
        return self._tune

    @tune.setter
    def tune(self, tune):
        """
        Tune setting for ffmpeg.

        :param tune: The tune of this EncodeConfig.
        :type: str
        """
        allowed_values = ["film", "animation", "grain", "stillimage", "fastdecode", "zerolatency", "psnr", "ssim"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and tune not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `tune` ({0}), must be one of {1}"  # noqa: E501
                .format(tune, allowed_values)
            )

        self._tune = tune

    @property
    def vcodec(self):
        """
        Video codec.

        :return: The vcodec of this EncodeConfig. 
        :rtype: str
        """
        return self._vcodec

    @vcodec.setter
    def vcodec(self, vcodec):
        """
        Video codec.

        :param vcodec: The vcodec of this EncodeConfig.
        :type: str
        """
        allowed_values = ["h264", "hevc"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and vcodec not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `vcodec` ({0}), must be one of {1}"  # noqa: E501
                .format(vcodec, allowed_values)
            )

        self._vcodec = vcodec

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, EncodeConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, EncodeConfig):
            return True

        return self.to_dict() != other.to_dict()



import json

from ..openapi.tator_openapi.models import CreateResponse
from ..util.get_api import get_api
from ..util.get_parser import get_parser

def parse_args():
    parser = get_parser()
    parser.add_argument('--project', type=int, help="Unique integer identifying a project.")
    parser.add_argument('--media_type', type=int, help="Unique integer identifying a media type.")
    parser.add_argument('--section', type=str, help="Name of section to upload to.")
    parser.add_argument('--name', type=str, help="Name of file.")
    parser.add_argument('--md5', type=str, help="md5 sum of file.")
    parser.add_argument('--gid', type=str, help="Upload group ID.")
    parser.add_argument('--uid', type=str, help="Upload unique ID.")
    parser.add_argument('--output', type=str, help="Where to dump media ID.")
    parser.add_argument('--attributes', type=str, help="Attributes for media")
    return parser.parse_args()

def create_media(host, token, project, media_type, section, name, md5, gid, uid, attributes=None):
    """ Creates a media object and returns the ID.

    :param host: Host URL.
    :param token: API token.
    :param project: Unique integer identifying a project.
    :param media_type: Unique integer identifying a media type.
    :param section: Section name.
    :param name: File name.
    :param md5: md5 sum of file.
    :param gid: Upload group ID.
    :param uid: Upload unique ID.
    """
    api = get_api(host, token)
    spec ={
        'type': media_type,
        'section': section,
        'name': name,
        'md5': md5,
        'gid': gid,
        'uid': uid
    }
    if attributes:
        spec.update({'attributes': json.loads(attributes)})
    response = api.create_media(project, media_spec=spec)

    assert isinstance(response, CreateResponse)
    media_id = response.id

    return media_id

if __name__ == '__main__':
    args = parse_args()
    media_id = create_media(args.host, args.token, args.project, args.media_type,
                            args.section, args.name, args.md5, args.gid, args.uid, args.attributes)
    with open(args.output, 'w') as f:
        f.write(str(media_id))



#!/usr/bin/env python

""" Prepares media for transcode."""

import argparse
import os
import json
import subprocess
import logging
from urllib.parse import urlparse

from progressbar import progressbar

from ..util import md5sum

from .create_media import create_media
from .determine_transcode import determine_transcode
from .make_thumbnails import make_thumbnails

logger = logging.getLogger(__name__)

def parse_args():
    parser = argparse.ArgumentParser(description='Full transcode pipeline on a directory of files.')
    parser.add_argument('--url', type=str, help='URL where original file is hosted.')
    parser.add_argument('--work_dir', type=str, help='Directory where info should be saved.')
    parser.add_argument('--host', type=str, default='https://www.tatorapp.com', help='Host URL.')
    parser.add_argument('--token', type=str, help='REST API token.')
    parser.add_argument('--project', type=int, help='Unique integer specifying project ID.')
    parser.add_argument('--type', type=int, help='Unique integer specifying a media type.')
    parser.add_argument('--name', type=str, help='Name of the media.')
    parser.add_argument('--section', type=str, help='Media section name.')
    parser.add_argument('--attributes', type=str, help="Attributes for media")
    parser.add_argument('--media_id', type=int, help="Existing media ID, if applicable",
                        default=-1)
    parser.add_argument('--gid', type=str, help="Upload group ID.")
    parser.add_argument('--uid', type=str, help="Upload unique ID.")
    parser.add_argument('--group_to', type=int, default=480,
                         help='Vertical resolutions below this will be transcoded with '
                              'multi-headed ffmpeg.')
    parser.add_argument('--size', type=int, help="Size in bytes of the uploaded file.")
    return parser.parse_args()

def get_file_paths(url, work_dir):
    name = os.path.basename(urlparse(url).path)
    paths = {
        'original': os.path.join(work_dir, name),
        'thumbnail': os.path.join(work_dir, 'thumbnail.jpg'),
        'thumbnail_gif': os.path.join(work_dir, 'thumbnail_gif.gif'),
        'media_id': os.path.join(work_dir, 'media_id.txt'),
        'workloads': os.path.join(work_dir, 'workloads.json'),
    }
    return paths

if __name__ == '__main__':
    # Parse arguments.
    args = parse_args()

    # Get file paths.
    paths = get_file_paths(args.url, args.work_dir)

    # Download the file.
    subprocess.run(['wget', '-O', paths['original'], args.url])

    # Get md5 for the file.
    md5 = md5sum(args.url, args.size)

    # Get base filename.
    if args.name:
        name = args.name
    else:
        name = os.path.basename(paths['original'])

    # Determine transcodes that need to be done.
    workloads = determine_transcode(args.host, args.token, args.type, paths['original'],
                                    args.group_to)
    for workload in workloads:
        workload['configs'] = ','.join(workload['configs'])
    with open(paths['workloads'], 'w') as f:
        json.dump(workloads, f)

    # Create the media object.
    if args.media_id == -1:
        media_id = create_media(args.host, args.token, args.project, args.type, args.section,
                                name, md5, args.gid, args.uid, args.attributes)
    else:
        media_id = args.media_id

    with open(paths['media_id'], 'w') as f:
        f.write(str(media_id))

    # Make thumbnails.
    make_thumbnails(args.host, args.token, media_id, paths['original'], paths['thumbnail'],
                    paths['thumbnail_gif'])





import logging
import os
import sys

import tator
from ._upload_file import _upload_file

logger = logging.getLogger(__name__)

def register_algorithm(
    host: str,
    token: str,
    project: int,
    manifest: str,
    algorithm_name: str,
    files_per_job: int,
    description: str='',
    cluster_id: int=None) -> None:
    """ Registers an algorithm argo workflow using the provided parameters

    This will upload the given manifest file, save it to tator, and the new
    server side URL will be used when registering the workflow. This may change
    the filename of the manifest file.

    Args:
        host: Host URL
        token: User token used for connecting to the host
        project: Unique identifier of project associated with algorithm
        manifest: Local path to argo workflow manifest file to upload, save,
            and register the algorithm with
        algorithm_name: Unique name of the workflow
        files_per_job: Number of media to process per workflow created
        description: Optional description of workflow
        cluster_id: Optional unique integer identifying the job cluster
    """

    # Create the interface
    tator_api = tator.get_api(host=host, token=token)
    user = tator_api.whoami()
    user_id = user.id

    # Upload the manifest file
    for progress, upload_info in _upload_file(tator_api, project, path=manifest):
        pass
    manifest_upload_url = tator_api.get_download_info(project, {'keys': [upload_info.key]})[0].url

    # Save the uploaded file using the save algorithm manifest endpoint
    spec = tator.models.AlgorithmManifestSpec(
        name=os.path.basename(manifest),
        upload_url=manifest_upload_url)
    response = tator_api.save_algorithm_manifest(project=project, algorithm_manifest_spec=spec)
            
    # Register the algorithm argo workflow
    spec = tator.models.Algorithm(
        name=algorithm_name,
        project=project,
        user=user_id,
        description=description,
        manifest=response.url,
        cluster=cluster_id,
        files_per_job=files_per_job)

    response = tator_api.register_algorithm(project=project, algorithm_spec=spec)

    log_msg = f"Algorithm registered with ID: {response.id}"
    logger.info(log_msg)



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class ColorMap(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'alpha_ranges': 'dict(str, list)',
        'default': 'object',
        'default_fill': 'Fill',
        'fill_map': 'dict(str, Fill)',
        'key': 'str',
        'map': 'dict(str, object)',
        'version': 'dict(str, object)'
    }

    attribute_map = {
        'alpha_ranges': 'alpha_ranges',
        'default': 'default',
        'default_fill': 'defaultFill',
        'fill_map': 'fillMap',
        'key': 'key',
        'map': 'map',
        'version': 'version'
    }

    def __init__(self, alpha_ranges=None, default=None, default_fill=None, fill_map=None, key=None, map=None, version=None, local_vars_configuration=None):  # noqa: E501
        """ColorMap - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._alpha_ranges = None
        self._default = None
        self._default_fill = None
        self._fill_map = None
        self._key = None
        self._map = None
        self._version = None
        self.discriminator = None

        if alpha_ranges is not None:
            self.alpha_ranges = alpha_ranges
        if default is not None:
            self.default = default
        if default_fill is not None:
            self.default_fill = default_fill
        if fill_map is not None:
            self.fill_map = fill_map
        if key is not None:
            self.key = key
        if map is not None:
            self.map = map
        if version is not None:
            self.version = version

    @property
    def alpha_ranges(self):
        """
        Map of attribute values to alpha level.

        :return: The alpha_ranges of this ColorMap. 
        :rtype: dict(str, list)
        """
        return self._alpha_ranges

    @alpha_ranges.setter
    def alpha_ranges(self, alpha_ranges):
        """
        Map of attribute values to alpha level.

        :param alpha_ranges: The alpha_ranges of this ColorMap.
        :type: dict(str, list)
        """

        self._alpha_ranges = alpha_ranges

    @property
    def default(self):
        """
        RGB array, RGBA array, or hex string.

        :return: The default of this ColorMap. 
        :rtype: object
        """
        return self._default

    @default.setter
    def default(self, default):
        """
        RGB array, RGBA array, or hex string.

        :param default: The default of this ColorMap.
        :type: object
        """

        self._default = default

    @property
    def default_fill(self):
        """

        :return: The default_fill of this ColorMap. 
        :rtype: Fill
        """
        return self._default_fill

    @default_fill.setter
    def default_fill(self, default_fill):
        """

        :param default_fill: The default_fill of this ColorMap.
        :type: Fill
        """

        self._default_fill = default_fill

    @property
    def fill_map(self):
        """
        Map of attribute values to fill types.

        :return: The fill_map of this ColorMap. 
        :rtype: dict(str, Fill)
        """
        return self._fill_map

    @fill_map.setter
    def fill_map(self, fill_map):
        """
        Map of attribute values to fill types.

        :param fill_map: The fill_map of this ColorMap.
        :type: dict(str, Fill)
        """

        self._fill_map = fill_map

    @property
    def key(self):
        """
        Attribute name.

        :return: The key of this ColorMap. 
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """
        Attribute name.

        :param key: The key of this ColorMap.
        :type: str
        """

        self._key = key

    @property
    def map(self):
        """
        Map of attribute values to colors.

        :return: The map of this ColorMap. 
        :rtype: dict(str, object)
        """
        return self._map

    @map.setter
    def map(self, map):
        """
        Map of attribute values to colors.

        :param map: The map of this ColorMap.
        :type: dict(str, object)
        """

        self._map = map

    @property
    def version(self):
        """
        Map of version IDs to colors.

        :return: The version of this ColorMap. 
        :rtype: dict(str, object)
        """
        return self._version

    @version.setter
    def version(self, version):
        """
        Map of version IDs to colors.

        :param version: The version of this ColorMap.
        :type: dict(str, object)
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ColorMap):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ColorMap):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class StateTypeUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'delete_child_localizations': 'bool',
        'description': 'str',
        'grouping_default': 'bool',
        'name': 'str',
        'visible': 'bool'
    }

    attribute_map = {
        'delete_child_localizations': 'delete_child_localizations',
        'description': 'description',
        'grouping_default': 'grouping_default',
        'name': 'name',
        'visible': 'visible'
    }

    def __init__(self, delete_child_localizations=False, description=None, grouping_default=True, name=None, visible=None, local_vars_configuration=None):  # noqa: E501
        """StateTypeUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._delete_child_localizations = None
        self._description = None
        self._grouping_default = None
        self._name = None
        self._visible = None
        self.discriminator = None

        if delete_child_localizations is not None:
            self.delete_child_localizations = delete_child_localizations
        if description is not None:
            self.description = description
        if grouping_default is not None:
            self.grouping_default = grouping_default
        if name is not None:
            self.name = name
        if visible is not None:
            self.visible = visible

    @property
    def delete_child_localizations(self):
        """
        True if child localizations should be deleted when this state is deleted. Localizations will only be deleted if they are not associated with another state. 

        :return: The delete_child_localizations of this StateTypeUpdate. 
        :rtype: bool
        """
        return self._delete_child_localizations

    @delete_child_localizations.setter
    def delete_child_localizations(self, delete_child_localizations):
        """
        True if child localizations should be deleted when this state is deleted. Localizations will only be deleted if they are not associated with another state. 

        :param delete_child_localizations: The delete_child_localizations of this StateTypeUpdate.
        :type: bool
        """

        self._delete_child_localizations = delete_child_localizations

    @property
    def description(self):
        """
        Description of the state type.

        :return: The description of this StateTypeUpdate. 
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Description of the state type.

        :param description: The description of this StateTypeUpdate.
        :type: str
        """

        self._description = description

    @property
    def grouping_default(self):
        """
        Whether to group elements in the UI by default.

        :return: The grouping_default of this StateTypeUpdate. 
        :rtype: bool
        """
        return self._grouping_default

    @grouping_default.setter
    def grouping_default(self, grouping_default):
        """
        Whether to group elements in the UI by default.

        :param grouping_default: The grouping_default of this StateTypeUpdate.
        :type: bool
        """

        self._grouping_default = grouping_default

    @property
    def name(self):
        """
        Name of the state type.

        :return: The name of this StateTypeUpdate. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the state type.

        :param name: The name of this StateTypeUpdate.
        :type: str
        """

        self._name = name

    @property
    def visible(self):
        """
        Whether this state type should be displayed.

        :return: The visible of this StateTypeUpdate. 
        :rtype: bool
        """
        return self._visible

    @visible.setter
    def visible(self, visible):
        """
        Whether this state type should be displayed.

        :param visible: The visible of this StateTypeUpdate.
        :type: bool
        """

        self._visible = visible

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StateTypeUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, StateTypeUpdate):
            return True

        return self.to_dict() != other.to_dict()



import os
import time
import tempfile
from textwrap import dedent

import tator
import uuid

def test_algorithm_cancel(host, token, project, image):
    ALGORITHM_NAME = f'Sleepy time {uuid.uuid1()}'

    # Register an algorithm that sleeps.
    workflow_spec = dedent("""\
apiVersion: argoproj.io/v1alpha1
kind: Workflow
metadata:
  generateName: sleep-
spec:
  entrypoint: sleep
  ttlStrategy:
    secondsAfterCompletion: 0
  templates:
  - name: sleep
    terminationGracePeriodSeconds: 0
    container:
      image: busybox
      command: [sleep]
      args: ["1000"]
      resources:
        limits:
          memory: 32Mi
          cpu: 100m
""")
    manifest_file = tempfile.NamedTemporaryFile(delete=False, mode='w')
    manifest_file.write(workflow_spec)
    manifest_file.close()
    tator.util.register_algorithm(
        host=host,
        token=token,
        project=project,
        manifest=manifest_file.name,
        algorithm_name=ALGORITHM_NAME,
        files_per_job=1,
        description='sleeps for testing purposes',
        cluster_id=None)
    os.remove(manifest_file.name)

    # Launch some workflows.
    tator_api = tator.get_api(host=host, token=token)
    spec = tator.models.AlgorithmLaunchSpec(
        algorithm_name=ALGORITHM_NAME,
        media_ids=[image])
    responses = [tator_api.algorithm_launch(project=project,
                                            algorithm_launch_spec=spec)
                 for _ in range(10)]

    # Cancel by UID.
    uid = responses[0].uid[0]
    job = tator_api.get_job(uid)
    assert isinstance(job, tator.models.Job)
    print(f"Cancelling job with uid {uid}...")
    response = tator_api.delete_job(uid)
    try:
        found = True
        response = tator_api.get_job(uid)
    except tator.exceptions.ApiException as exc:
        found = False
        assert exc.status == 404
    assert found == False

    # Cancel by GID.
    gid = responses[1].gid
    jobs = tator_api.get_job_list(project, gid=gid)
    assert isinstance(jobs[0], tator.models.Job)
    print(f"Cancelling job with gid {gid}...")
    tator_api.delete_job_list(project, gid=gid)
    jobs = tator_api.get_job_list(project, gid=gid)
    assert len(jobs) == 0

    # Cancel all jobs in project.
    jobs = tator_api.get_job_list(project)
    assert len(jobs) >= 8
    print("Cancelling all jobs in project...")
    tator_api.delete_job_list(project)
    jobs = tator_api.get_job_list(project)
    assert len(jobs) == 0




# coding: utf-8

# flake8: noqa
"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from tator.openapi.tator_openapi.models.affiliation import Affiliation
from tator.openapi.tator_openapi.models.affiliation_spec import AffiliationSpec
from tator.openapi.tator_openapi.models.affiliation_update import AffiliationUpdate
from tator.openapi.tator_openapi.models.algorithm import Algorithm
from tator.openapi.tator_openapi.models.algorithm_launch import AlgorithmLaunch
from tator.openapi.tator_openapi.models.algorithm_launch_spec import AlgorithmLaunchSpec
from tator.openapi.tator_openapi.models.algorithm_manifest import AlgorithmManifest
from tator.openapi.tator_openapi.models.algorithm_manifest_spec import AlgorithmManifestSpec
from tator.openapi.tator_openapi.models.algorithm_parameter import AlgorithmParameter
from tator.openapi.tator_openapi.models.algorithm_spec import AlgorithmSpec
from tator.openapi.tator_openapi.models.analysis import Analysis
from tator.openapi.tator_openapi.models.analysis_spec import AnalysisSpec
from tator.openapi.tator_openapi.models.archive_config import ArchiveConfig
from tator.openapi.tator_openapi.models.attribute_bulk_update import AttributeBulkUpdate
from tator.openapi.tator_openapi.models.attribute_type import AttributeType
from tator.openapi.tator_openapi.models.attribute_type_delete import AttributeTypeDelete
from tator.openapi.tator_openapi.models.attribute_type_spec import AttributeTypeSpec
from tator.openapi.tator_openapi.models.attribute_type_update import AttributeTypeUpdate
from tator.openapi.tator_openapi.models.audio_definition import AudioDefinition
from tator.openapi.tator_openapi.models.autocomplete_service import AutocompleteService
from tator.openapi.tator_openapi.models.bad_request_response import BadRequestResponse
from tator.openapi.tator_openapi.models.bookmark import Bookmark
from tator.openapi.tator_openapi.models.bookmark_spec import BookmarkSpec
from tator.openapi.tator_openapi.models.bookmark_update import BookmarkUpdate
from tator.openapi.tator_openapi.models.clone_media_spec import CloneMediaSpec
from tator.openapi.tator_openapi.models.color_map import ColorMap
from tator.openapi.tator_openapi.models.create_list_response import CreateListResponse
from tator.openapi.tator_openapi.models.create_response import CreateResponse
from tator.openapi.tator_openapi.models.credentials import Credentials
from tator.openapi.tator_openapi.models.download_info import DownloadInfo
from tator.openapi.tator_openapi.models.download_info_spec import DownloadInfoSpec
from tator.openapi.tator_openapi.models.email_attachment_spec import EmailAttachmentSpec
from tator.openapi.tator_openapi.models.email_spec import EmailSpec
from tator.openapi.tator_openapi.models.encode_config import EncodeConfig
from tator.openapi.tator_openapi.models.favorite import Favorite
from tator.openapi.tator_openapi.models.favorite_spec import FavoriteSpec
from tator.openapi.tator_openapi.models.favorite_update import FavoriteUpdate
from tator.openapi.tator_openapi.models.fill import Fill
from tator.openapi.tator_openapi.models.image_definition import ImageDefinition
from tator.openapi.tator_openapi.models.job import Job
from tator.openapi.tator_openapi.models.job_node import JobNode
from tator.openapi.tator_openapi.models.leaf import Leaf
from tator.openapi.tator_openapi.models.leaf_spec import LeafSpec
from tator.openapi.tator_openapi.models.leaf_suggestion import LeafSuggestion
from tator.openapi.tator_openapi.models.leaf_type import LeafType
from tator.openapi.tator_openapi.models.leaf_type_spec import LeafTypeSpec
from tator.openapi.tator_openapi.models.leaf_type_update import LeafTypeUpdate
from tator.openapi.tator_openapi.models.leaf_update import LeafUpdate
from tator.openapi.tator_openapi.models.localization import Localization
from tator.openapi.tator_openapi.models.localization_spec import LocalizationSpec
from tator.openapi.tator_openapi.models.localization_type import LocalizationType
from tator.openapi.tator_openapi.models.localization_type_spec import LocalizationTypeSpec
from tator.openapi.tator_openapi.models.localization_type_update import LocalizationTypeUpdate
from tator.openapi.tator_openapi.models.localization_update import LocalizationUpdate
from tator.openapi.tator_openapi.models.media import Media
from tator.openapi.tator_openapi.models.media_files import MediaFiles
from tator.openapi.tator_openapi.models.media_next import MediaNext
from tator.openapi.tator_openapi.models.media_prev import MediaPrev
from tator.openapi.tator_openapi.models.media_spec import MediaSpec
from tator.openapi.tator_openapi.models.media_stats import MediaStats
from tator.openapi.tator_openapi.models.media_type import MediaType
from tator.openapi.tator_openapi.models.media_type_spec import MediaTypeSpec
from tator.openapi.tator_openapi.models.media_type_update import MediaTypeUpdate
from tator.openapi.tator_openapi.models.media_update import MediaUpdate
from tator.openapi.tator_openapi.models.membership import Membership
from tator.openapi.tator_openapi.models.membership_spec import MembershipSpec
from tator.openapi.tator_openapi.models.membership_update import MembershipUpdate
from tator.openapi.tator_openapi.models.message_response import MessageResponse
from tator.openapi.tator_openapi.models.multi_definition import MultiDefinition
from tator.openapi.tator_openapi.models.not_found_response import NotFoundResponse
from tator.openapi.tator_openapi.models.notify_spec import NotifySpec
from tator.openapi.tator_openapi.models.organization import Organization
from tator.openapi.tator_openapi.models.organization_spec import OrganizationSpec
from tator.openapi.tator_openapi.models.project import Project
from tator.openapi.tator_openapi.models.project_spec import ProjectSpec
from tator.openapi.tator_openapi.models.project_update import ProjectUpdate
from tator.openapi.tator_openapi.models.resolution_config import ResolutionConfig
from tator.openapi.tator_openapi.models.s3_storage_config import S3StorageConfig
from tator.openapi.tator_openapi.models.section import Section
from tator.openapi.tator_openapi.models.section_spec import SectionSpec
from tator.openapi.tator_openapi.models.section_update import SectionUpdate
from tator.openapi.tator_openapi.models.state import State
from tator.openapi.tator_openapi.models.state_merge_update import StateMergeUpdate
from tator.openapi.tator_openapi.models.state_spec import StateSpec
from tator.openapi.tator_openapi.models.state_trim_update import StateTrimUpdate
from tator.openapi.tator_openapi.models.state_type import StateType
from tator.openapi.tator_openapi.models.state_type_spec import StateTypeSpec
from tator.openapi.tator_openapi.models.state_type_update import StateTypeUpdate
from tator.openapi.tator_openapi.models.state_update import StateUpdate
from tator.openapi.tator_openapi.models.temporary_file import TemporaryFile
from tator.openapi.tator_openapi.models.temporary_file_spec import TemporaryFileSpec
from tator.openapi.tator_openapi.models.token import Token
from tator.openapi.tator_openapi.models.transcode import Transcode
from tator.openapi.tator_openapi.models.transcode_spec import TranscodeSpec
from tator.openapi.tator_openapi.models.upload_completion_spec import UploadCompletionSpec
from tator.openapi.tator_openapi.models.upload_info import UploadInfo
from tator.openapi.tator_openapi.models.upload_part import UploadPart
from tator.openapi.tator_openapi.models.user import User
from tator.openapi.tator_openapi.models.user_update import UserUpdate
from tator.openapi.tator_openapi.models.version import Version
from tator.openapi.tator_openapi.models.version_spec import VersionSpec
from tator.openapi.tator_openapi.models.version_update import VersionUpdate
from tator.openapi.tator_openapi.models.video_clip import VideoClip
from tator.openapi.tator_openapi.models.video_definition import VideoDefinition



def clone_localization_type(src_api, src_type_id, dest_project, media_type_mapping, dest_api=None):
    """ Clone localization type.

    This can be used to clone localization types from one project to another or from one
    host to another.

    Example for different host:

    .. code-block:: python

        src_api = tator.get_api(host, token)
        dest_api = tator.get_api(other_host, other_token)
        src_type_id = 1
        dest_project = 1
        media_type_mapping = {1: 10}
        response = tator.util.clone_localization_type(src_api, src_type_id, dest_project,
                                                      media_type_mapping, dest_api)
        print(response.message)

    Example for same host:

    .. code-block:: python

        api = tator.get_api(host, token)
        src_type_id = 1
        dest_project = 1
        response = tator.util.clone_localization_type(src_api, src_type_id, dest_project,
                                                      media_type_mapping)
        print(response.message)

    :param src_api: :class:`tator.TatorApi` object corresponding to source host or only
        host if this is a clone on same host.
    :param src_type_id: Unique integer identifying localization type to clone.
    :param dest_project: Unique integer identifying destination project.
    :param media_type_mapping: Dictionary mapping source media types to destination media types.
    :param dest_api: :class:`tator.TatorApi` object corresponding to destination host.
    :returns: Response from localization type creation request.
    """
    type_obj = src_api.get_localization_type(src_type_id)
    spec = {'name': type_obj.name,
            'description': type_obj.description,
            'dtype': type_obj.dtype,
            'grouping_default': type_obj.grouping_default,
            'color_map': type_obj.color_map,
            'line_width': type_obj.line_width,
            'visible': type_obj.visible,
            'attribute_types': type_obj.attribute_types}
    dest_media_types = []
    for src_media_type in type_obj.media:
        if src_media_type in media_type_mapping:
            dest_media_types.append(media_type_mapping[src_media_type])
        else:
            raise ValueError(f"Media type mapping does not contain source media ID {src_media_type}!")
    spec['media_types'] = dest_media_types
    if dest_api is None:
        dest_api = src_api
    return dest_api.create_localization_type(dest_project, localization_type_spec=spec)



import tator
from uuid import uuid1

def test_clone_section_same_host(host, token, project, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator_api.create_section(project=project, section_spec={'name': str(uuid1())})
    section = response.id
    response = tator.util.clone_section(tator_api, section, clone_project)
    assert(isinstance(response, tator.models.CreateResponse))

def test_clone_section_different_host(host, token, project, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator_api.create_section(project=project, section_spec={'name': str(uuid1())})
    section = response.id
    response = tator.util.clone_section(tator_api, section, clone_project, tator_api)
    assert(isinstance(response, tator.models.CreateResponse))




import os
import subprocess
import shutil

from distutils.cmd import Command
from setuptools import setup, find_packages  # noqa: H301
from setuptools.command.dist_info import dist_info
import requests
import yaml
import json

REQUIRES = ["urllib3 >= 1.15", "six >= 1.10", "certifi", "python-dateutil",
            "requests", "pyyaml", "progressbar2"]

SCHEMA_FILENAME = 'schema.yaml'
CONFIG_FILENAME = 'config.json'

def get_version():
    with open(CONFIG_FILENAME, 'r') as f:
        config = json.load(f)
    return config['packageVersion']

def remove_oneof(data):
    """ Removes oneOf key from a dict and recursively calls this function on
        other dict values.
    """
    if 'oneOf' in data:
        del data['oneOf']
    for key in data:
        if isinstance(data[key], dict):
            remove_oneof(data[key])
    return data

class NoAliasDumper(yaml.Dumper):
    def ignore_aliases(self, data):
        return True

def codegen():
    """ Fetches a schema from tatorapp.com if one does not exist, then 
        use openapi-generator to generate openapi code from it.
    """
    # Retrieve schema if it does not exist.
    if not os.path.exists(SCHEMA_FILENAME):
        response = requests.get("https://www.tatorapp.com/schema")
        assert response.status_code == 200
        with open(SCHEMA_FILENAME, 'wb') as f:
            f.write(response.content)

    # Remove any oneOf entries from the schema, as they are not handled
    # well by openapi generator.
    with open(SCHEMA_FILENAME, 'r') as f:
        schema = yaml.load(f)
    schema = remove_oneof(schema)
    with open(SCHEMA_FILENAME, 'w') as f:
        yaml.dump(schema, f, Dumper=NoAliasDumper)

    # Get the git SHA ID.
    cmd = ['git', 'rev-parse', 'HEAD']
    git_rev = subprocess.check_output(cmd).strip().decode('utf-8')

    # Generate code using openapi generator docker image.
    pwd = os.path.dirname(os.path.abspath(__file__))
    cmd = [
        'docker', 'run', '-it', '--rm',
        '-v', f"{pwd}:/pwd",
        '-v', f"{pwd}/out:/out",
        'openapitools/openapi-generator-cli:v4.3.1', 'generate',
        '-c', f'/pwd/{CONFIG_FILENAME}',
        '-i', f'/pwd/{SCHEMA_FILENAME}',
        '-g', 'python',
        '-o', f'/out/tator-py-{git_rev}',
        '-t', '/pwd/templates',
    ]
    subprocess.run(cmd, check=True)

    # Remove the schema.
    os.remove(SCHEMA_FILENAME)

    # Copy relevant directories into openapi.
    out_dir = os.path.join(pwd, 'tator/openapi')
    os.makedirs(out_dir, exist_ok=True)
    for subpath in ['README.md', 'tator_openapi', 'docs']:
        src = f'{pwd}/out/tator-py-{git_rev}/{subpath}'
        dst = os.path.join(out_dir, f'{subpath}')
        if os.path.isfile(src):
            shutil.copy(src, dst)
        else:
            if os.path.exists(dst):
                shutil.rmtree(dst)
            shutil.copytree(src, dst)
    pwd = os.path.dirname(os.path.abspath(__file__))

    # need to delete from within docker
    cmd = [
        'docker', 'run', '-it', '--rm',
        '-v', f"{pwd}/out:/out",
         'openapitools/openapi-generator-cli:v4.3.1',
        'rm', '-fr',
        '/out/*'
    ]
    subprocess.run(cmd, check=True)

codegen()
setup(
    name="tator",
    version=get_version(),
    description="Python client for Tator",
    author="CVision AI, Inc.",
    author_email="info@cvisionai.com",
    url="https://github.com/cvisionai/tator-py",
    keywords=["OpenAPI", "OpenAPI-Generator", "Tator"],
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    long_description="""\
    Python client for Tator.
    """
)



import os

import tator

def test_download_media(host, token, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)
    video_path = f'/tmp/{video_obj.name}'
    if os.path.exists(video_path):
        os.remove(video_path)
    for progress in tator.download_media(tator_api, video_obj, video_path):
        print(f"Media download progress: {progress}%")
    assert os.path.exists(video_path)
    assert os.stat(video_path).st_size == 2266862
